from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSService(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSService, self).__init__()

        self.options = {
            # Read-write values
            'cachetype': '',
            'servername': '',
            'downstateflush': '',
            'maxreq': '',
            'maxbandwidth': '',
            'svrtimeout': '',
            'port': '',
            'clttimeout': '',
            'servicetype': '',
            'cacheable': '',
            'maxclient': '',
            'ipaddress': '',
            'delay': '',
            'graceful': '',
            'usip': '',
            'rtspsessionidremap': '',
            'cleartextport': '',
            'monthreshold': '',
            'accessdown': '',
            'serverid': '',
            'cka': '',
            'name': '',
            'newname': '',
            'sp': '',
            'weight': '',
            'cip': '',
            'monitor_name_svc': '',
            'cipheader': '',
            'useproxyport': '',
            'sc': '',
            'cmp': '',
            'tcpb': '',
            'state': '',
            'httpprofilename': '',
            'tcpprofilename': '',
            'comment': '',

            # Readonly values
            'policyname': '',
            'monstatparam2': '',
            'monstatparam3': '',
            'stateupdatereason': '',
            'serviceconftype': '',
            'gslb': '',
            'svrstate': '',
            'monstatcode': '',
            'timesincelaststatechange': '',
            'tickssincelaststatechange': '',
            'responsetime': '',
            'statechangetimesec': '',
            'statechangetimemsec': '',
            'failedprobes': '',
            'totalfailedprobes': '',
            'totalprobes': '',
        }

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options:
                    self.options[key] = json_data[key]

        self.resourcetype = NSService.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "service"

    # Getters and setters for configurable options
    def get_name(self):
        """
        The name of the service.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['name']

    def set_name(self, name):
        """
        The name of the service.
        Default value: 0
        Minimum length =  1.
        """
        self.options['name'] = name

    def get_newname(self):
        """
        The new name of the service.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['newname']

    def set_newname(self, name):
        """
        The new name of the service.
        Default value: 0
        Minimum length =  1.
        """
        self.options['newname'] = name

    def get_cachetype(self):
        """
        The cache type option supported by the cache server.
        Default value: 0
        """
        return self.options['cachetype']

    def set_cachetype(self, cachetype):
        """
        The cache type option supported by the cache server.
        Default value: 0
        """
        self.options['cachetype'] = cachetype

    def get_servername(self):
        """
        The name of the server for which a service will be added.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['servername']

    def set_servername(self, servername):
        """
        The name of the server for which a service will be added.
        Default value: 0
        Minimum length =  1.
        """
        self.options['servername'] = servername

    def get_downstateflush(self):
        """
        Perform delayed clean up of connections on this service.
        Default value: ENABLED
        """
        return self.options['downstateflush']

    def set_downstateflush(self, downstateflush):
        """
        Perform delayed clean up of connections on this service.
        Default value: ENABLED
        """
        self.options['downstateflush'] = downstateflush

    def get_maxreq(self):
        """
        The maximum number of requests that can be sent on a persistent connection to the service.
        Minimum value =  0
        Maximum value =  65535
        """
        return self.options['maxreq']

    def set_maxreq(self, maxreq):
        """
        The maximum number of requests that can be sent on a persistent connection to the service.
        Minimum value =  0
        Maximum value =  65535
        """
        self.options['maxreq'] = maxreq

    def get_maxbandwidth(self):
        """
        The maximum bandwidth in kbps allowed for the service.
        Default value: 0
        Minimum value =  0
        Maximum value =  4294967287
        """
        return self.options['maxbandwidth']

    def set_maxbandwidth(self, maxbandwidth):
        """
        The maximum bandwidth in kbps allowed for the service.
        Default value: 0
        Minimum value =  0
        Maximum value =  4294967287
        """
        self.options['maxbandwidth'] = maxbandwidth

    def get_svrtimeout(self):
        """
        The idle time (in seconds) after which the server connection is terminated.
        Minimum value =  0
        Maximum value =  31536000
        """
        return self.options['svrtimeout']

    def set_svrtimeout(self, svrtimeout):
        """
        The idle time (in seconds) after which the server connection is terminated.
        Minimum value =  0
        Maximum value =  31536000
        """
        self.options['svrtimeout'] = svrtimeout

    def get_port(self):
        """
        The port number to be used for the service.
        Default value: 0
        Range 1 - 65535.
        """
        return self.options['port']

    def set_port(self, port):
        """
        The port number to be used for the service.
        Default value: 0
        Range 1 - 65535.
        """
        self.options['port'] = port

    def get_weight(self):
        """
        The weight for the specified monitor.
        Default value: 0
        Minimum value =  1
        Maximum value =  100
        """
        return self.options['weight']

    def set_weight(self, weight):
        """
        The weight for the specified monitor.
        Default value: 0
        Minimum value =  1
        Maximum value =  100
        """
        self.options['weight'] = weight

    def get_monitor_name_svc(self):
        """
        The monitor name bound to the selected service.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['monitor_name_svc']

    def set_monitor_name_svc(self, monitornamesvc):
        """
        The monitor name bound to the selected service.
        Default value: 0
        Minimum length =  1.
        """
        self.options['monitor_name_svc'] = monitornamesvc

    def get_clttimeout(self):
        """
        The idle time (in seconds) after which the client connection is terminated.
        Minimum value =  0
        Maximum value =  31536000
        """
        return self.options['clttimeout']

    def set_clttimeout(self, clttimeout):
        """
        The idle time (in seconds) after which the client connection is terminated.
        Minimum value =  0
        Maximum value =  31536000
        """
        self.options['clttimeout'] = clttimeout

    def get_servicetype(self):
        """
        The type of service. The supported protocols are:
        HTTP - To load balance web servers and provide connection multiplexing, latency improvement, and other content and TCP protection benefits for HTTP traffic.
        FTP - To load balance FTP servers. In FTP mode, the system provides TCP protection benefits, protection against SYN attacks, and surge protection.
        TCP - To host any other TCP protocols that are not HTTP, FTP, NNTP, or SSL. In TCP mode, the system provides TCPprotection benefits, protection against SYN attack, and surge protection.
        UDP - To load balance servers with UDP-based services (other than DNS).
        SSL - To provide end-to-end encryption and SSL acceleration.
        SSL_BRIDGE - To load balance SSL servers.
        SSL_TCP - To offload SSL traffic for TCP applications.
        NNTP - To load balance NNTP servers.
        DNS - To load balance DNS servers.
        ADNS: To create an authoritative DNS service.
        ANY - To load balance a service type not listed above (for example, for IP traffic when load balancing firewalls).
        Note:  The NNTP service is for cache redirection.

        Default value: 0
        """
        return self.options['servicetype']

    def set_servicetype(self, servicetype):
        """
        The type of service. The supported protocols are:
        HTTP - To load balance web servers and provide connection multiplexing, latency improvement, and other content and TCP protection benefits for HTTP traffic.
        FTP - To load balance FTP servers. In FTP mode, the system provides TCP protection benefits, protection against SYN attacks, and surge protection.
        TCP - To host any other TCP protocols that are not HTTP, FTP, NNTP, or SSL. In TCP mode, the system provides TCPprotection benefits, protection against SYN attack, and surge protection.
        UDP - To load balance servers with UDP-based services (other than DNS).
        SSL - To provide end-to-end encryption and SSL acceleration.
        SSL_BRIDGE - To load balance SSL servers.
        SSL_TCP - To offload SSL traffic for TCP applications.
        NNTP - To load balance NNTP servers.
        DNS - To load balance DNS servers.
        ADNS: To create an authoritative DNS service.
        ANY - To load balance a service type not listed above (for example, for IP traffic when load balancing firewalls).
        Note:  The NNTP service is for cache redirection.

        Default value: 0
        """
        self.options['servicetype'] = servicetype

    def get_cacheable(self):
        """
        The virtual server (used in load balancing or content switching) routes a request to the virtual server
        (used in transparent cache redirection) on the same system before sending it to the configured servers.
        The virtual server used for transparent cache redirection determines if the request is directed to the
        cache servers or configured servers.
        Note:  This argument is disabled by default. Do not specify this argument if a -cacheType cacheType is specified.

        Default value: NO
        """
        return self.options['cacheable']

    def set_cacheable(self, cacheable):
        """
        The virtual server (used in load balancing or content switching) routes a request to the virtual server
        (used in transparent cache redirection) on the same system before sending it to the configured servers.
        The virtual server used for transparent cache redirection determines if the request is directed to the
        cache servers or configured servers.
        Note:  This argument is disabled by default. Do not specify this argument if a -cacheType cacheType is specified.

        Default value: NO
        """
        self.options['cacheable'] = cacheable

    def get_maxclient(self):
        """
        The maximum number of open connections to the service.
        Minimum value =  0
        Maximum value =  4294967294
        """
        return self.options['maxclient']

    def set_maxclient(self, maxclient):
        """
        The maximum number of open connections to the service.
        Minimum value =  0
        Maximum value =  4294967294
        """
        self.options['maxclient'] = maxclient

    def get_ipaddress(self):
        """
        The new IP address of the service.
        Default value: 0
        """
        return self.options['ipaddress']

    def set_ipaddress(self, ipaddress):
        """
        The new IP address of the service.
        Default value: 0
        """
        self.options['ipaddress'] = ipaddress

    def get_delay(self):
        """
        The time allowed (in seconds) for a graceful shutdown. During this period, new connections and requests
        continue to be sent to the service for clients who already have persistent sessions on the system.
        Connections or requests from fresh or new clients who do not yet have a persistence sessions on the
        NetScaler system are not sent to the service. Instead, they are load balanced among other available
        services. After the delay time has passed, no new requests or connections are sent to the service.

        Default value: 0
        """
        return self.options['delay']

    def set_delay(self, delay):
        """
        The time allowed (in seconds) for a graceful shutdown. During this period, new connections and requests
        continue to be sent to the service for clients who already have persistent sessions on the system.
        Connections or requests from fresh or new clients who do not yet have a persistence sessions on the
        NetScaler system are not sent to the service. Instead, they are load balanced among other available
        services. After the delay time has passed, no new requests or connections are sent to the service.

        Default value: 0
        """
        self.options['delay'] = delay

    def set_graceful(self, graceful):
        """
        Indicates graceful shutdown of the service. System will wait for all outstanding
        connections to this service to be closed before disabling the service.
        Default value: NO
        """
        self.options['graceful'] = graceful

    def get_graceful(self):
        """
        Indicates graceful shutdown of the service. System will wait for all outstanding
        connections to this service to be closed before disabling the service.
        Default value: NO
        """
        return self.options['graceful']

    def get_usip(self):
        """
        The use of client's IP Address option to the source IP Address while connecting to this server.
        By default, the system uses a mapped IP address for its server connection; however, you can use this
        option to tell the system to use the client's IP address when it communicates with the server.
        """
        return self.options['usip']

    def set_usip(self, usip):
        """
        The use of client's IP Address option to the source IP Address while connecting to this server.
        By default, the system uses a mapped IP address for its server connection; however, you can use this
        option to tell the system to use the client's IP address when it communicates with the server.
        """
        self.options['usip'] = usip

    def get_rtspsessionidremap(self):
        """
        Use this parameter to enable mapping of RTSP sessionid.
        Default value: OFF
        """
        return self.options['rtspsessionidremap']

    def set_rtspsessionidremap(self, rtspsessionidremap):
        """
        Use this parameter to enable mapping of RTSP sessionid.
        Default value: OFF
        """
        self.options['rtspsessionidremap'] = rtspsessionidremap

    def get_cleartextport(self):
        """
        The clear text port number where clear text data is sent. Used with SSL offload service.
        Default value: 0
        Minimum value =  1
        """
        return self.options['cleartextport']

    def set_cleartextport(self, cleartextport):
        """
        The clear text port number where clear text data is sent. Used with SSL offload service.
        Default value: 0
        Minimum value =  1
        """
        self.options['cleartextport'] = cleartextport

    def get_monthreshold(self):
        """
        The monitoring threshold.
        Default value: 0
        Minimum value =  0
        Maximum value =  65535
        """
        return self.options['monthreshold']

    def set_monthreshold(self, monthreshold):
        """
        The monitoring threshold.
        Default value: 0
        Minimum value =  0
        Maximum value =  65535
        """
        self.options['monthreshold'] = monthreshold

    def get_accessdown(self):
        """
        The option to allow access to disabled or down services. If enabled, all packets to this service are
        bridged.  If disabled, they are dropped.
        Default value: NO
        """
        return self.options['accessdown']

    def set_accessdown(self, accessdown):
        """
        The option to allow access to disabled or down services. If enabled, all packets to this service are
        bridged.  If disabled, they are dropped.
        Default value: NO
        """
        self.options['accessdown'] = accessdown

    def get_serverid(self):
        """
        The  identifier for the service. This is used when the persistency type is set to Custom Server ID.
        Default value: 0
        """
        return self.options['serverid']

    def set_serverid(self, serverid):
        """
        The  identifier for the service. This is used when the persistency type is set to Custom Server ID.
        Default value: 0
        """
        self.options['serverid'] = serverid

    def get_state(self):
        """
        The state of the service after it is added.
        Default value: ENABLED
        """
        return self.options['state']

    def set_state(self, state):
        """
        The state of the service after it is added.
        Default value: ENABLED
        """
        self.options['state'] = state

    def get_cka(self):
        """
        The state of the Client Keep-Alive feature for the service.
        """
        return self.options['cka']

    def set_cka(self, cka):
        """
        The state of the Client Keep-Alive feature for the service.
        """
        self.options['cka'] = cka

    def get_sp(self):
        """
        The state of Surge protection for the the service.
        """
        return self.options['sp']

    def set_sp(self, sp):
        """
        The state of Surge protection for the the service.
        """
        self.options['sp'] = sp

    def get_cip(self):
        """
        The Client IP header insertion option for the service.
        """
        return self.options['cip']

    def set_cip(self, cip):
        """
        The Client IP header insertion option for the service.
        """
        self.options['cip'] = cip

    def get_cipheader(self):
        """
        The client IP header. If client IP insertion is enabled and the client IP header is not specified,
        then the value set by the ###set ns config### command will be used as the Client IP header.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['cipheader']

    def set_cipheader(self, cipheader):
        """
        The client IP header. If client IP insertion is enabled and the client IP header is not specified,
        then the value set by the ###set ns config### command will be used as the Client IP header.
        Default value: 0
        Minimum length =  1.
        """
        self.options['cipheader'] = cipheader

    def get_useproxyport(self):
        """
        When USIP is enabled, based on the setting of this variable proxy port or the client port will be used
        as the source port for the backend connection.
        """
        return self.options['useproxyport']

    def set_useproxyport(self, useproxyport):
        """
        When USIP is enabled, based on the setting of this variable proxy port or the client port will be used
        as the source port for the backend connection.
        """
        self.options['useproxyport'] = useproxyport

    def get_sc(self):
        """
        The state of SureConnect for the service.
        Note:  This parameter is supported for legacy purposes only. It has no effect on this system, and its
        only valid value is OFF.
        Default value: OFF
        """
        return self.options['sc']

    def set_sc(self, sc):
        """
        The state of SureConnect for the service.
        Note:  This parameter is supported for legacy purposes only. It has no effect on this system, and its
        only valid value is OFF.
        Default value: OFF
        """
        self.options['sc'] = sc

    def get_cmp(self):
        """
        The state of the HTTP Compression feature for the service.
        """
        return self.options['cmp']

    def set_cmp(self, cmp):
        """
        The state of the HTTP Compression feature for the service.
        """
        self.options['cmp'] = cmp

    def get_tcpb(self):
        """
        The state of the TCP Buffering feature for the service.
        """
        return self.options['tcpb']

    def set_tcpb(self, tcpb):
        """
        The state of the TCP Buffering feature for the service.
        """
        self.options['tcpb'] = tcpb

    def set_tcpprofilename(self, tcpprofilename):
        """
        The name of the TCP profile.
        Default value: 0
        Minimum length =  1.
        Maximum length =  127.
        """
        self.options['tcpprofilename'] = tcpprofilename

    def get_tcpprofilename(self):
        """
        The name of the TCP profile.
        Default value: 0
        Minimum length =  1.
        Maximum length =  127.
        """
        return self.options['tcpprofilename']

    def set_httpprofilename(self, httpprofilename):
        """
        Name of the HTTP profile.
        Default value: 0
        Minimum length =  1.
        Maximum length =  127.
        """
        self.options['httpprofilename'] = httpprofilename

    def get_httpprofilename(self):
        """
        Name of the HTTP profile.
        Default value: 0
        Minimum length =  1.
        Maximum length =  127.
        """
        return self.options['httpprofilename']

    def set_comment(self, comment):
        """
        Comments associated with this server.
        Default value: 0
        """
        self.options['comment'] = comment

    def get_comment(self):
        """
        Comments associated with this server.
        Default value: 0
        """
        return self.options['comment']

    # Read-only option getters
    def get_svrstate(self):
        """
        The state of the service.
        Default value: 0
        """
        return self.options['svrstate']

    def get_policyname(self):
        """
        The name of the policyname for which this service is bound.
        Default value: 0
        """
        return self.options['policyname']

    def get_monstatparam(self):
        """
        First parameter for use with message code.
        Default value: 0
        """
        return self.options['monstatparam2']

    def get_monstatparam2(self):
        """
        Second parameter for use with message code.
        Default value: 0
        """
        return self.options['monstatparam2']

    def get_monstatparam3(self):
        """
        Third parameter for use with message code.
        Default value: 0
        """
        return self.options['monstatparam3']

    def get_stateupdatereason(self):
        """
        Checks state update reason on the secondary node.
        Default value: 0
        """
        return self.options['stateupdatereason']

    def get_serviceconftype(self):
        """
        The configuration type of the service.
        Default value: 0
        """
        return self.options['serviceconftype']

    def get_gslb(self):
        """
        The GSLB option for the corresponding virtual server.
        Default value: 0
        """
        return self.options['gslb']

    def get_monstatcode(self):
        """
        The code indicating the monitor response.
        Default value: 0
        """
        return self.options['monstatcode']

    def get_timesincelaststatechange(self):
        """
        Time in milliseconds since the last state change.
        Default value: 0
        """
        return self.options['timesincelaststatechange']

    def get_tickssincelaststatechange(self):
        """
        Time in 10 millisecond ticks since the last state change.
        Default value: 0
        """
        return self.options['tickssincelaststatechange']

    def get_responsetime(self):
        """
        Response time of this monitor.
        Default value: 0
        """
        return self.options['responsetime']

    def get_statechangetimesec(self):
        """
        Time when last state change happened. Seconds part.
        Default value: 0
        """
        return self.options['statechangetimesec']

    def get_statechangetimemsec(self):
        """
        Time at which last state change happened. Milliseconds part.
        Default value: 0
        """
        return self.options['statechangetimemsec']

    # Operations methods
    @staticmethod
    def disable(nitro, service):
        """
        Use this API to disable service.
        """
        __service = NSService()
        __service.set_name(service.get_name())
        __service.set_delay(service.get_delay())
        __service.set_graceful(service.get_graceful())
        return __service.perform_operation(nitro, "disable")

    @staticmethod
    def enable(nitro, service):
        """
        Use this API to enable service.
        """
        __service = NSService()
        __service.set_name(service.get_name())
        return __service.perform_operation(nitro, "enable")

    @staticmethod
    def rename(nitro, service):
        """
        Use this API to rename service.
        """
        __service = NSService()
        __service.set_name(service.get_name())
        __service.set_newname(service.get_newname())
        return __service.perform_operation(nitro, "rename")

    @staticmethod
    def get(nitro, service):
        """
        Use this API to fetch service resource of given name.
        """
        __service = NSService()
        __service.set_name(service.get_name())
        __service.get_resource(nitro)
        return __service

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured service resources.
        """
        __url = nitro.get_url() + NSService.get_resourcetype()
        __json_services = nitro.get(__url).get_response_field(NSService.get_resourcetype())
        __services = []
        for json_service in __json_services:
            __services.append(NSService(json_service))
        return __services

    @staticmethod
    def add(nitro, service):
        """
        Use this API to add service.
        """
        __service = NSService()
        __service.set_name(service.get_name())
        __service.set_ipaddress(service.get_ipaddress())
        __service.set_servername(service.get_servername())
        __service.set_servicetype(service.get_servicetype())
        __service.set_port(service.get_port())
        __service.set_cleartextport(service.get_cleartextport())
        __service.set_cachetype(service.get_cachetype())
        __service.set_maxclient(service.get_maxclient())
        __service.set_maxreq(service.get_maxreq())
        __service.set_cacheable(service.get_cacheable())
        __service.set_cip(service.get_cip())
        __service.set_cipheader(service.get_cipheader())
        __service.set_usip(service.get_usip())
        __service.set_useproxyport(service.get_useproxyport())
        __service.set_sc(service.get_sc())
        __service.set_sp(service.get_sp())
        __service.set_rtspsessionidremap(service.get_rtspsessionidremap())
        __service.set_clttimeout(service.get_clttimeout())
        __service.set_svrtimeout(service.get_svrtimeout())
        __service.set_serverid(service.get_serverid())
        __service.set_cka(service.get_cka())
        __service.set_tcpb(service.get_tcpb())
        __service.set_maxbandwidth(service.get_maxbandwidth())
        __service.set_accessdown(service.get_accessdown())
        __service.set_monthreshold(service.get_monthreshold())
        __service.set_state(service.get_state())
        __service.set_downstateflush(service.get_downstateflush())
        __service.set_tcpprofilename(service.get_tcpprofilename())
        __service.set_httpprofilename(service.get_httpprofilename())
        __service.set_comment(service.get_comment())
        return __service.add_resource(nitro)

    @staticmethod
    def update(nitro, service):
        """
        Use this API to add service.
        """
        __service = NSService()
        __service.set_name(service.get_name())
        __service.set_ipaddress(service.get_ipaddress())
        __service.set_maxclient(service.get_maxclient())
        __service.set_maxreq(service.get_maxreq())
        __service.set_cacheable(service.get_cacheable())
        __service.set_cip(service.get_cip())
        __service.set_cipheader(service.get_cipheader())
        __service.set_usip(service.get_usip())
        __service.set_useproxyport(service.get_useproxyport())
        __service.set_sc(service.get_sc())
        __service.set_sp(service.get_sp())
        __service.set_rtspsessionidremap(service.get_rtspsessionidremap())
        __service.set_clttimeout(service.get_clttimeout())
        __service.set_svrtimeout(service.get_svrtimeout())
        __service.set_serverid(service.get_serverid())
        __service.set_cka(service.get_cka())
        __service.set_tcpb(service.get_tcpb())
        __service.set_cmp(service.get_cmp())
        __service.set_maxbandwidth(service.get_maxbandwidth())
        __service.set_accessdown(service.get_accessdown())
        __service.set_monthreshold(service.get_monthreshold())
        __service.set_weight(service.get_weight())
        __service.set_monitor_name_svc(service.get_monitor_name_svc())
        __service.set_downstateflush(service.get_downstateflush())
        __service.set_tcpprofilename(service.get_tcpprofilename())
        __service.set_httpprofilename(service.get_httpprofilename())
        __service.set_comment(service.get_comment())
        return __service.update_resource(nitro)

    @staticmethod
    def delete(nitro, service):
        """
        Use this API to delete service of a given name.
        """
        __service = NSService()
        __service.set_name(service.get_name())
        nsresponse = __service.delete_resource(nitro)
        return nsresponse



from nsbaseresource import NSBaseResource


class NSAcl(NSBaseResource):

    # Configuration for ACL entry resource.

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSAcl, self).__init__()
        self.options = {
            'aclname': '',
            'aclaction': '',
            'srcip': '',
            'srcipop': '',
            'srcipval': '',
            'srcport': '',
            'srcportop': '',
            'srcportval': '',
            'destip': '',
            'destipop': '',
            'destipval': '',
            'destport': '',
            'destportop': '',
            'destportval': '',
            'ttl': '',
            'srcmac': '',
            'protocol': '',
            'protocolnumber': '',
            'vlan': '',
            'Interface': '',
            'established': '',
            'icmptype': '',
            'icmpcode': '',
            'priority': '',
            'state': '',
            'logstate': '',
            'ratelimit': '',
            'hits': '',
            'kernelstate': '',
            '__count': ''
        }

        self.resourcetype = NSAcl.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "nsacl"

    def set_aclname(self, aclname):
        """
        The alphanumeric name of the ACL.
        Default value: 0
        Minimum length =  1.
        """
        self.options['aclname'] = aclname

    def get_aclname(self):
        """
        The action associated with the ACL.
        Default value: 0
        """
        return self.options['aclname']

    def set_aclaction(self, aclaction):
        """
        The action associated with the ACL.
        Default value: 0
        """
        self.options['aclaction'] = aclaction

    def get_aclaction(self):
        """
        The action associated with the ACL.
        Default value: 0
        """
        return self.options['aclaction']

    def set_srcip(self, srcip):
        """
        The source IP address (range).
        Default value: 0
        """
        self.options['srcip'] = srcip

    def get_srcip(self):
        """
        The source IP address (range).
        Default value: 0
        """
        return self.options['srcip']

    def set_srcipop(self, srcipop):
        """
        Logical operator.
        Default value: 0
        """
        self.options['srcipop'] = srcipop

    def get_srcipop(self):
        """
        Logical operator.
        Default value: 0
        """
        return self.options['srcipop']

    def set_srcipval(self, srcipval):
        """
        The source IP address (range).
        Default value: 0
        """
        self.options['srcipval'] = srcipval

    def get_srcipval(self):
        """
        The source IP address (range).
        Default value: 0
        """
        return self.options['srcipval']

    def set_srcport(self, srcport):
        """
        The source Port (range).
        Default value: 0
        """
        self.options['srcport'] = srcport

    def get_srcport(self):
        """
        The source Port (range).
        Default value: 0
        """
        return self.options['srcport']

    def set_srcportop(self, srcportop):
        """
        Logical operator.
        Default value: 0
        """
        self.options['srcportop'] = srcportop

    def get_srcportop(self):
        """
        Logical operator.
        Default value: 0
        """
        return self.options['srcportop']

    def set_srcportval(self, srcportval):
        """
        The source Port (range).
        Default value: 0
        Maximum length =  65535.
        """
        self.options['srcportval'] = srcportval

    def get_srcportval(self):
        """
        The source Port (range).
        Default value: 0
        Maximum length =  65535.
        """
        return self.options['srcportval']

    def set_destip(self, destip):
        """
        The destination IP address (range).
        Default value: 0
        """
        self.options['destip'] = destip

    def get_destip(self):
        """
        The destination IP address (range).
        Default value: 0
        """
        return self.options['destip']

    def set_destipop(self, destipop):
        """
        Logical operator.
        Default value: 0
        """
        self.options['destipop'] = destipop

    def get_destipop(self):
        """
        Logical operator.
        Default value: 0
        """
        return self.options['destipop']

    def set_destipval(self, destipval):
        """
        The destination IP address (range).
        Default value: 0
        """
        self.options['destipval'] = destipval

    def get_destipval(self):
        """
        The destination IP address (range).
        Default value: 0
        """
        return self.options['destipval']

    def set_destport(self, destport):
        """
        The destination Port (range).
        Default value: 0
        """
        self.options['destport'] = destport

    def get_destport(self):
        """
        The destination Port (range).
        Default value: 0
        """
        return self.options['destport']

    def set_destportop(self, destportop):
        """
        Logical operator.
        Default value: 0
        """
        self.options['destportop'] = destportop

    def get_destportop(self):
        """
        Logical operator.
        Default value: 0
        """
        return self.options['destportop']

    def set_destportval(self, destportval):
        """
        The destination Port (range).
        Default value: 0
        Maximum length =  65535.
        """
        self.options['destportval'] = destportval

    def get_destportval(self):
        """
        The destination Port (range).
        Default value: 0
        Maximum length =  65535.
        """
        return self.options['destportval']

    def set_ttl(self, ttl):
        """
        The time to expire this ACL(in seconds).
        Default value: 0
        Minimum value =  1
        Maximum value =  0x7FFFFFFF
        """
        self.options['ttl'] = ttl

    def get_ttl(self):
        """
        The time to expire this ACL(in seconds).
        Default value: 0
        Minimum value =  1
        Maximum value =  0x7FFFFFFF
        """
        return self.options['ttl']

    def set_srcmac(self, srcmac):
        """
        The source MAC address.
        Default value: 0
        """
        self.options['srcmac'] = srcmac

    def get_srcmac(self):
        """
        The source MAC address.
        Default value: 0
        """
        return self.options['srcmac']

    def set_protocol(self, protocol):
        """
        The IP protocol name.
        Default value: 0
        """
        self.options['protocol'] = protocol

    def get_protocol(self):
        """
        The IP protocol name.
        Default value: 0
        """
        return self.options['protocol']

    def set_protocolnumber(self, protocolnumber):
        """
        The IP protocol number (decimal).
        Default value: 0
        Minimum value =  1
        Maximum value =  255
        """
        self.options['protocolnumber'] = protocolnumber

    def get_protocolnumber(self):
        """
        The IP protocol number (decimal).
        Default value: 0
        Minimum value =  1
        Maximum value =  255
        """
        return self.options['protocolnumber']

    def set_vlan(self, vlan):
        """
        The VLAN number.
        Default value: 0
        Minimum value =  1
        Maximum value =  4094
        """
        self.options['vlan'] = vlan

    def get_vlan(self):
        """
        The VLAN number.
        Default value: 0
        Minimum value =  1
        Maximum value =  4094
        """
        return self.options['vlan']

    def set_Interface(self, Interface):
        """
        The physical interface.
        Default value: 0
        """
        self.options['Interface'] = Interface

    def get_Interface(self):
        """
        The physical interface.
        Default value: 0
        """
        return self.options['Interface']

    def set_established(self, established):
        """
        This argument indicates that the ACL should be used for TCP response traffic only.
        """
        self.options['established'] = established

    def get_established(self):
        """
        This argument indicates that the ACL should be used for TCP response traffic only.
        """
        return self.options['established']

    def set_icmptype(self, icmptype):
        """
        The ICMP message type.
        Default value: 65536
        Minimum value =  0
        Maximum value =  255
        """
        self.options['icmptype'] = icmptype

    def get_icmptype(self):
        """
        The ICMP message type.
        Default value: 65536
        Minimum value =  0
        Maximum value =  255
        """
        return self.options['icmptype']

    def set_icmpcode(self, icmpcode):
        """
        The ICMP message code.
        Default value: 65536
        Minimum value =  0
        Maximum value =  255
        """
        self.options['icmpcode'] = icmpcode

    def get_icmpcode(self):
        """
        The ICMP message code.
        Default value: 65536
        Minimum value =  0
        Maximum value =  255
        """
        return self.options['icmpcode']

    def set_priority(self, priority):
        """
        The priority of the ACL.
        Default value: 0
        Minimum value =  1
        Maximum value =  10240
        """
        self.options['priority'] = priority

    def get_priority(self):
        """
        The priority of the ACL.
        Default value: 0
        Minimum value =  1
        Maximum value =  10240
        """
        return self.options['priority']

    def set_state(self, state):
        """
        The state of the ACL.
        Default value: ENABLED
        """
        self.options['state'] = state

    def get_state(self):
        """
        The state of the ACL.
        Default value: ENABLED
        """
        return self.options['state']

    def set_logstate(self, logstate):
        """
        The logging state of the ACL.
        Default value: DISABLED
        """
        self.options['logstate'] = logstate

    def get_logstate(self):
        """
        The logging state of the ACL.
        Default value: DISABLED
        """
        return self.options['logstate']

    def set_ratelimit(self, ratelimit):
        """
        log message rate limit for acl rule.
        Default value: 100
        Minimum value =  1
        Maximum value =  10000
        """
        self.options['ratelimit'] = ratelimit

    def get_ratelimit(self):
        """
        log message rate limit for acl rule.
        Default value: 100
        Minimum value =  1
        Maximum value =  10000
        """
        return self.options['ratelimit']

    def get_hits(self):
        """
        The hits of this ACL.
        Default value: 0
        """
        return self.options['hits']

    def get_kernelstate(self):
        """
        The commit status of the ACL.
        Default value: 0
        """
        return self.options['kernelstate']

    # Operations methods

    @staticmethod
    def add(nitro, nsacl):
        """
        Use this API to add nsacl resources.
        """
        __nsacl = NSAcl()
        __nsacl.set_aclname(nsacl.get_aclname())
        __nsacl.set_aclaction(nsacl.get_aclaction())
        __nsacl.set_srcip(nsacl.get_srcip())
        __nsacl.set_srcipop(nsacl.get_srcipop())
        __nsacl.set_srcipval(nsacl.get_srcipval())
        __nsacl.set_srcport(nsacl.get_srcport())
        __nsacl.set_srcportop(nsacl.get_srcportop())
        __nsacl.set_srcportval(nsacl.get_srcportval())
        __nsacl.set_destip(nsacl.get_destip())
        __nsacl.set_destipop(nsacl.get_destipop())
        __nsacl.set_destipval(nsacl.get_destipval())
        __nsacl.set_destport(nsacl.get_destport())
        __nsacl.set_destportop(nsacl.get_destportop())
        __nsacl.set_destportval(nsacl.get_destportval())
        __nsacl.set_ttl(nsacl.get_ttl())
        __nsacl.set_srcmac(nsacl.get_srcmac())
        __nsacl.set_protocol(nsacl.get_protocol())
        __nsacl.set_protocolnumber(nsacl.get_protocolnumber())
        __nsacl.set_vlan(nsacl.get_vlan())
        __nsacl.set_Interface(nsacl.get_Interface())
        __nsacl.set_established(nsacl.get_established())
        __nsacl.set_icmptype(nsacl.get_icmptype())
        __nsacl.set_icmpcode(nsacl.get_icmpcode())
        __nsacl.set_priority(nsacl.get_priority())
        __nsacl.set_state(nsacl.get_state())
        __nsacl.set_logstate(nsacl.get_logstate())
        __nsacl.set_ratelimit(nsacl.get_ratelimit())
        return __nsacl.add_resource(nitro)

    @staticmethod
    def delete(nitro, nsacl):
        """
        Use this API to delete nsacl resources.
        """
        __nsacl = NSAcl()
        __nsacl.set_aclname(nsacl.get_aclname())
        nsresponse = __nsacl.delete_resource(nitro, nsacl.get_aclname())
        return nsresponse

    @staticmethod
    def update(nitro, nsacl):
        """
        Use this API to update nsacl resources.
        """
        __nsacl = NSAcl()
        __nsacl.set_aclname(nsacl.get_aclname())
        __nsacl.set_aclaction(nsacl.get_aclaction())
        __nsacl.set_srcip(nsacl.get_srcip())
        __nsacl.set_srcipop(nsacl.get_srcipop())
        __nsacl.set_srcipval(nsacl.get_srcipval())
        __nsacl.set_srcport(nsacl.get_srcport())
        __nsacl.set_srcportop(nsacl.get_srcportop())
        __nsacl.set_srcportval(nsacl.get_srcportval())
        __nsacl.set_destip(nsacl.get_destip())
        __nsacl.set_destipop(nsacl.get_destipop())
        __nsacl.set_destipval(nsacl.get_destipval())
        __nsacl.set_destport(nsacl.get_destport())
        __nsacl.set_destportop(nsacl.get_destportop())
        __nsacl.set_destportval(nsacl.get_destportval())
        __nsacl.set_srcmac(nsacl.get_srcmac())
        __nsacl.set_protocol(nsacl.get_protocol())
        __nsacl.set_protocolnumber(nsacl.get_protocolnumber())
        __nsacl.set_icmptype(nsacl.get_icmptype())
        __nsacl.set_icmpcode(nsacl.get_icmpcode())
        __nsacl.set_vlan(nsacl.get_vlan())
        __nsacl.set_Interface(nsacl.get_Interface())
        __nsacl.set_priority(nsacl.get_priority())
        __nsacl.set_state(nsacl.get_state())
        __nsacl.set_logstate(nsacl.get_logstate())
        __nsacl.set_ratelimit(nsacl.get_ratelimit())
        return __nsacl.update_resource(nitro)

#    @staticmethod
#    def unset(nitro, nsacl):
#        """ generated source for method unset """
#        unsetresource = nsacl()
#        unsetresource.aclname = aclname
#        return unsetresource.unset_resource(client, args)

    @staticmethod
    def enable(nitro, nsacl):
        """
        Use this API to enable a acl of given name.
        """
        __nsacl = NSAcl()
        __nsacl.set_aclname(nsacl.get_aclname())
        return __nsacl.perform_operation(nitro, "enable")

    @staticmethod
    def disable(nitro, nsacl):
        """
        Use this API to disable a acl of given name.
        """
        __nsacl = NSAcl()
        __nsacl.set_aclname(nsacl.get_aclname())
        return __nsacl.perform_operation(nitro, "disable")

    @staticmethod
    def get(nitro, nsacl):
        """
        Use this API to fetch nsacl resource of given name.
        """
        __nsacl = NSAcl()
        __nsacl.set_aclname(nsacl.get_aclname())
        __nsacl.get_resource(nitro, object_name=__nsacl.get_aclname())
        return __nsacl

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured nsacl resources.
        """
        __url = nitro.get_url() + NSAcl.get_resourcetype()
        __json_nsacls = nitro.get(__url).get_response_field(NSAcl.get_resourcetype())
        __nsacls = []
        for json_nsacl in __json_nsacls:
            __nsacls.append(NSAcl(json_nsacl))
        return __nsacls



from nsbaseresource import NSBaseResource


class NSLBVServer(NSBaseResource):

    # Configuration for Load Balancing Virtual Server resource.

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSLBVServer, self).__init__()
        self.options = {'name': '',
                        'servicetype': '',
                        'ipv46': '',
                        'ippattern': '',
                        'ipmask': '',
                        'port': '',
                        'range': '',
                        'persistencetype': '',
                        'timeout': '',
                        'persistencebackup': '',
                        'backuppersistencetimeout': '',
                        'lbmethod': '',
                        'hashlength': '',
                        'netmask': '',
                        'v6netmasklen': '',
                        'rule': '',
                        'listenpolicy': '',
                        'listenpriority': '',
                        'resrule': '',
                        'persistmask': '',
                        'v6persistmasklen': '',
                        'pq': '',
                        'sc': '',
                        'rtspnat': '',
                        'm': '',
                        'tosid': '',
                        'datalength': '',
                        'dataoffset': '',
                        'sessionless': '',
                        'state': '',
                        'connfailover': '',
                        'redirurl': '',
                        'cacheable': '',
                        'clttimeout': '',
                        'somethod': '',
                        'sopersistence': '',
                        'sopersistencetimeout': '',
                        'sothreshold': '',
                        'redirectportrewrite': '',
                        'downstateflush': '',
                        'backupvserver': '',
                        'disableprimaryondown': '',
                        'insertvserveripport': '',
                        'vipheader': '',
                        'authenticationhost': '',
                        'authentication': '',
                        'authn401': '',
                        'authnvsname': '',
                        'push': '',
                        'pushvserver': '',
                        'pushlabel': '',
                        'pushmulticlients': '',
                        'tcpprofilename': '',
                        'httpprofilename': '',
                        'comment': '',
                        'weight': '',
                        'servicename': '',
                        'redirurlflags': '',
                        'newname': '',
                        'value': '',
                        'ipmapping': '',
                        'type': '',
                        'curstate': '',
                        'effectivestate': '',
                        'status': '',
                        'lbrrreason': '',
                        'redirect': '',
                        'precedence': '',
                        'homepage': '',
                        'dnsvservername': '',
                        'domain': '',
                        'cachevserver': '',
                        'health': '',
                        'gotopriorityexpression': '',
                        'ruletype': '',
                        'groupname': '',
                        'cookiedomain': '',
                        'map': '',
                        'gt2gb': '',
                        'thresholdvalue': '',
                        'bindpoint': '',
                        'invoke': '',
                        'labeltype': '',
                        'labelname': '',
                        'version': '',
                        'totalservices': '',
                        'activeservices': '',
                        'statechangetimeseconds': '',
                        'statechangetimemsec': '',
                        'tickssincelaststatechange': '',
                        '__count': ''}

        self.resourcetype = NSLBVServer.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "lbvserver"

    def set_name(self, name):
        """
        The name of the load balancing virtual server being added.
        Default value: 0
        Minimum length =  1.
        """
        self.options['name'] = name

    def get_name(self):
        """
        The name of the load balancing virtual server being added.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['name']

    def set_servicetype(self, servicetype):
        """
        The service type.
        Default value: 0
        """
        self.options['servicetype'] = servicetype

    def get_servicetype(self):
        """
        The service type.
        Default value: 0
        """
        return self.options['servicetype']

    def set_ipv46(self, ipv46):
        """
        The IP address of the virtual server.
        Default value: 0
        """
        self.options['ipv46'] = ipv46

    def get_ipv46(self):
        """
        The IP address of the virtual server.
        Default value: 0
        """
        return self.options['ipv46']

    def set_ippattern(self, ippattern):
        """
        The IP Pattern of the virtual server.
        Default value: 0
        """
        self.options['ippattern'] = ippattern

    def get_ippattern(self):
        """
        The IP Pattern of the virtual server.
        Default value: 0
        """
        return self.options['ippattern']

    def set_ipmask(self, ipmask):
        """
        The IP Mask of the virtual server IP Pattern.
        """
        self.options['ipmask'] = ipmask

    def get_ipmask(self):
        """
        The IP Mask of the virtual server IP Pattern.
        """
        return self.options['ipmask']

    def set_port(self, port):
        """
        A port number for the virtual server.
        Default value: 0
        Range 1 - 65535.
        """
        self.options['port'] = port

    def get_port(self):
        """
        A port number for the virtual server.
        Default value: 0
        Range 1 - 65535.
        """
        return self.options['port']

    def set_range(self, myrange):
        """
        The IP range for the network vserver.
        Default value: 1
        Minimum value =  1
        Maximum value =  254
        """
        self.options['range'] = myrange

    def get_range(self):
        """
        The IP range for the network vserver.
        Default value: 1
        Minimum value =  1
        Maximum value =  254
        """
        return self.options['range']

    def set_persistencetype(self, persistencetype):
        self.options['persistencetype'] = persistencetype

    def get_persistencetype(self):
        return self.options['persistencetype']

    def set_timeout(self, timeout):
        """
        The time period for which the persistence is in effect for a specific client.
        The value ranges from 2 to 1440 minutes.
        Default value: 2
        Minimum value =  0
        Maximum value =  1440
        """
        self.options['timeout'] = timeout

    def get_timeout(self):
        """
        The time period for which the persistence is in effect for a specific client.
        The value ranges from 2 to 1440 minutes.
        Default value: 2
        Minimum value =  0
        Maximum value =  1440
        """
        return self.options['timeout']

    def set_persistencebackup(self, persistencebackup):
        """
        Use this parameter to specify a backup persistence type for the virtual server.
        The Backup persistence option is used when the primary configured persistence
        mechanism on virtual server fails.
        The <persistenceBacup> parameter can take one of the following options:
           SOURCEIP
           NONE.
        Default value: 0
        """
        self.options['persistencebackup'] = persistencebackup

    def get_persistencebackup(self):
        """
        Use this parameter to specify a backup persistence type for the virtual server.
        The Backup persistence option is used when the primary configured persistence
        mechanism on virtual server fails.
        The <persistenceBacup> parameter can take one of the following options:
           SOURCEIP
           NONE.
        Default value: 0
        """
        return self.options['persistencebackup']

    def set_backuppersistencetimeout(self, backuppersistencetimeout):
        """
        The maximum time backup persistence is in effect for a specific client.
        Default value: 2
        Minimum value =  2
        Maximum value =  1440
        """
        self.options['backuppersistencetimeout'] = backuppersistencetimeout

    def get_backuppersistencetimeout(self):
        """
        The maximum time backup persistence is in effect for a specific client.
        Default value: 2
        Minimum value =  2
        Maximum value =  1440
        """
        return self.options['backuppersistencetimeout']

    def set_lbmethod(self, lbmethod):
        """
        The load balancing method for the virtual server. The valid options for this parameter are:
        ROUNDROBIN, LEASTCONNECTION, LEASTRESPONSETIME, URLHASH, DOMAINHASH, DESTINATIONIPHASH, SOURCEIPHASH,
        SRCIPDESTIPHASH,LEASTBANDWIDTH, LEASTPACKETS, TOKEN, SRCIPDESTIPHASH, CUSTOMLOAD, SRCIPSRCPORTHASH,
        LRTM, CALLIDHASSH.
        When the load balancing policy is configured as:
             ROUNDROBIN - When configured, the system distributes incoming requests to each server in rotation,
               regardless of the load. When different weights are assigned to services then weighted round robin
               occurs and requests go to services according to how much weighting has been set.

             LEASTCONNECTION (default value)- When configured, the system selects the service that has the least
               number of connections. For TCP, HTTP, HTTPS and SSL_TCP services the least number of connections
                includes:
               Established, active connections to a service. Connection reuse applies to HTTP and HTTPS.
               Hence the count
               includes only those connections which have outstanding HTTP or HTTPS requests, and does
               not include inactive, reusable connections.
               Connections to a service waiting in the Surge Queue, which exists only if the Surge Protection
                feature is enabled.
        For UDP services the least number of connections includes:
               The number of sessions between client and a physical service. These sessions are the logical,
               time-based entities, created on first arriving UDP packet. If configured, weights are taken into
               account when server selection is performed.

             LEASTRESPONSETIME - When configured, the system selects the service with the minimum average
               response time. The response time is the time interval taken when a request is sent to a service
                and
               first response packet comes back from the service, that is Time to First Byte (TTFB).
             URLHASH - The system selects the service based on the hashed value of the incoming URL.
               To specify the number of bytes of the URL that is used to calculate the hash value use the
                optional
               argument [-hashLength <positive_integer>] in either the add lb vserver or set lb vserver CLI
                command.
               The default value is 80.

             DOMAINHASH - When configured with this load balancing method, the system selects the service
                based on
               the hashed value of the domain name in the HTTP request. The domain name is taken either from
               the incoming URL or from the Host header of the HTTP request.
        Note:   The system defaults to LEASTCONNECTION if the request does not contain a domain name.
        If the domain name appears in both the URL and the host header, the system gives preference to
         the URL domain.
        #
               DESTINATIONIPHASH - The system selects the service based on the hashed value of the destination
                IP address in the TCP IP header.
               SOURCEIPHASH - The system selects the service based on the hashed value of the client's
                IP address in the IP header.
               LEASTBANDWIDTH - The system selects the service that is currently serving the least traffic,
                measured in megabits per second.
               LEASTPACKETS - The system selects the service that is currently serving the lowest
                number of packets per second.
               Token -The system selects the service based on the value, calculated from a token, extracted
                from the client's request
               (location and size of the token is configurable). For subsequent requests with the same token,
                the systems will select the same physical server.
               SRCIPDESTIPHASH - The system selects the service based on the hashed value of the client's
                SOURCE IP and DESTINATION IP address in the TCP IP header.
               CUSTOMLOAD - The system selects the service based on the it load which was determined by the
                LOAD monitors bound to the service.
               SRCIPSRCPORTHASH - The system selects the service based on the hashed value of the client's
               SOURCE IP and SOURCE PORT in the TCP/UDP+IP header.
               LRTM - When configured, the system selects the service with least response time learned through
                probing(number of active
               connections taken into account in addition to the response time).
               CALLIDHASSH - The system selects the service based on the hashed value of SIP callid.
        Default value: LEASTCONNECTION
        """
        self.options['lbmethod'] = lbmethod

    def get_lbmethod(self):
        """
        The load balancing method for the virtual server. The valid options for this parameter are:
        ROUNDROBIN, LEASTCONNECTION, LEASTRESPONSETIME, URLHASH, DOMAINHASH, DESTINATIONIPHASH, SOURCEIPHASH,
        SRCIPDESTIPHASH,LEASTBANDWIDTH, LEASTPACKETS, TOKEN, SRCIPDESTIPHASH, CUSTOMLOAD, SRCIPSRCPORTHASH,
        LRTM, CALLIDHASSH.
        When the load balancing policy is configured as:
               ROUNDROBIN - When configured, the system distributes incoming requests to each server
               in rotation, regardless of the load. When different weights are assigned to services then
               weighted round robin occurs and requests go to services according to how much
               weighting has been set.
               LEASTCONNECTION (default value)- When configured, the system selects the service that has the
               least number of connections. For TCP, HTTP, HTTPS and SSL_TCP services the least number of
               connections includes:
               Established, active connections to a service. Connection reuse applies to HTTP and HTTPS. Hence
               the count includes only those connections which have outstanding HTTP or HTTPS requests,
               and does not include inactive, reusable connections.
               Connections to a service waiting in the Surge Queue, which exists only if the Surge Protection
                feature is enabled.
        For UDP services the least number of connections includes:
               The number of sessions between client and a physical service. These sessions are the logical,
                time-based entities, created on first arriving UDP packet. If configured, weights are taken
                 into account when server selection is performed.
               LEASTRESPONSETIME - When configured, the system selects the service with the minimum average
                response time. The response time is the time interval taken when a request is sent to a service
                 and first response packet comes back from the service, that is Time to First Byte (TTFB).
               URLHASH - The system selects the service based on the hashed value of the incoming URL.To specify
                the number of bytes of the URL that is used to calculate the hash value use the optional
                argument [-hashLength <positive_integer>] in either the add lb vserver or set lb vserver
                CLI command. The default value is 80.
               DOMAINHASH - When configured with this load balancing method, the system selects the service
               based on the hashed value of the domain name in the HTTP request. The domain name is taken
               either from the incoming URL or from the Host header of the HTTP request.
        Note:   The system defaults to LEASTCONNECTION if the request does not contain a domain name.
        If the domain name appears in both the URL and the host header, the system gives preference to
        the URL domain.
        #
               DESTINATIONIPHASH - The system selects the service based on the hashed value of the destination
                IP address in the TCP IP header.
               SOURCEIPHASH - The system selects the service based on the hashed value of the client's IP
               address in the IP header.
               LEASTBANDWIDTH - The system selects the service that is currently serving the least traffic,
               measured in megabits per second.
               LEASTPACKETS - The system selects the service that is currently serving the lowest number of
               packets per second.
               Token -The system selects the service based on the value, calculated from a token, extracted
               from the client's request (location and size of the token is configurable). For subsequent
               requests with the same token, the systems will select the same physical server.
               SRCIPDESTIPHASH - The system selects the service based on the hashed value of the client's
               SOURCE IP and DESTINATION IP address in the TCP IP header.
               CUSTOMLOAD - The system selects the service based on the it load which was determined by
               the LOAD monitors bound to the service.
               SRCIPSRCPORTHASH - The system selects the service based on the hashed value of
               the client's SOURCE IP and SOURCE PORT in the TCP/UDP+IP header.
               LRTM - When configured, the system selects the service with least response time learned through
               probing(number of active connections taken into account in addition to the response time).
               CALLIDHASSH - The system selects the service based on the hashed value of SIP callid.
        Default value: LEASTCONNECTION
        """
        return self.options['lbmethod']

    def set_hashlength(self, hashlength):
        """
        This parameter is used when the URLHASH or DOMAINHASH policy is set. Enter the number of bytes
        that will be hashed in the URL or DOMAIN name. Valid values are from 1 to 4096 bytes.
        Default value: 80
        Minimum value =  1
        Maximum value =  4096
        """
        self.options['hashlength'] = hashlength

    def get_hashlength(self):
        """
        This parameter is used when the URLHASH or DOMAINHASH policy is set. Enter the number of bytes
        that will be hashed in the URL or DOMAIN name. Valid values are from 1 to 4096 bytes.
        Default value: 80
        Minimum value =  1
        Maximum value =  4096
        """
        return self.options['hashlength']

    def set_netmask(self, netmask):
        """
        Use this parameter to set the DESTINATIONIPHASH or SOURCEIPHASH policy.
        Enter the netmask to be used in modifying the destination or source IP address or network.
        Minimum length =  1.
        """
        self.options['netmask'] = netmask

    def get_netmask(self):
        """
        Use this parameter to set the DESTINATIONIPHASH or SOURCEIPHASH policy.
        Enter the netmask to be used in modifying the destination or source IP address or network.
        Minimum length =  1.
        """
        return self.options['netmask']

    def set_v6netmasklen(self, v6netmasklen):
        """
        The Network mask. Use this parameter if you are setting the DESTINATIONIPHASH or SOURCEIPHASH policy.
        Enter the netmask to be used in modifying the destination or source IP address or network.
        Default value: 128
        Minimum value =  1
        Maximum value =  128
        """
        self.options['v6netmasklen'] = v6netmasklen

    def get_v6netmasklen(self):
        """
        The Network mask. Use this parameter if you are setting the DESTINATIONIPHASH or SOURCEIPHASH policy.
        Enter the netmask to be used in modifying the destination or source IP address or network.
        Default value: 128
        Minimum value =  1
        Maximum value =  128
        """
        return self.options['v6netmasklen']

    def set_rule(self, rule):
        """
        Use this parameter to specify the string value used to set the RULE persistence type.
        The string can be either an existing rule name (configured using add rule command) or else it can be
        an in-line expression with a maximum of 256 characters.
        Default value: "none"
        """
        self.options['rule'] = rule

    def get_rule(self):
        """
        Use this parameter to specify the string value used to set the RULE persistence type.
        The string can be either an existing rule name (configured using add rule command) or else it can be
        an in-line expression with a maximum of 256 characters.
        Default value: "none"
        """
        return self.options['rule']

    def set_listenpolicy(self, listenpolicy):
        """
        Use this parameter to specify the listen policy for LB Vserver.
        The string can be either an existing expression name (configured using add policy expression command)
        or else it can be an in-line expression with a maximum of 1500 characters.
        Default value: "none"
        """
        self.options['listenpolicy'] = listenpolicy

    def get_listenpolicy(self):
        """
        Use this parameter to specify the listen policy for LB Vserver.
        The string can be either an existing expression name (configured using add policy expression command)
        or else it can be an in-line expression with a maximum of 1500 characters.
        Default value: "none"
        """
        return self.options['listenpolicy']

    def set_listenpriority(self, listenpriority):
        """
        Use this parameter to specify the priority for listen policy of LB Vserver.
        Default value: 101
        Minimum value =  0
        Maximum value =  100
        """
        self.options['listenpriority'] = listenpriority

    def get_listenpriority(self):
        """
        Use this parameter to specify the priority for listen policy of LB Vserver.
        Default value: 101
        Minimum value =  0
        Maximum value =  100
        """
        return self.options['listenpriority']

    def set_resrule(self, resrule):
        """
        Use this parameter to specify the expression to be used in response for RULE persistence type.
        The string  is an in-line expression with a maximum of 1500 characters.
        Default value: "none"
        """
        self.options['resrule'] = resrule

    def get_resrule(self):
        """
        Use this parameter to specify the expression to be used in response for RULE persistence type.
        The string  is an in-line expression with a maximum of 1500 characters.
        Default value: "none"
        """
        return self.options['resrule']

    def set_persistmask(self, persistmask):
        """
        Use this parameter to specify if the persistency is IP based. This parameter is Optional.
        Minimum length =  1.
        """
        self.options['persistmask'] = persistmask

    def get_persistmask(self):
        """
        Use this parameter to specify if the persistency is IP based. This parameter is Optional.
        Minimum length =  1.
        """
        return self.options['persistmask']

    def set_v6persistmasklen(self, v6persistmasklen):
        """
        The persistence mask. Use this parameter if you are using IP based persistence type on a ipv6 vserver.
        Default value: 128
        Minimum value =  1
        Maximum value =  128
        """
        self.options['v6persistmasklen'] = v6persistmasklen

    def get_v6persistmasklen(self):
        """
        The persistence mask. Use this parameter if you are using IP based persistence type on a ipv6 vserver.
        Default value: 128
        Minimum value =  1
        Maximum value =  128
        """
        return self.options['v6persistmasklen']

    def set_pq(self, pq):
        """
        Use this parameter to enable priority queuing on the specified virtual server.
        Default value: OFF
        """
        self.options['pq'] = pq

    def get_pq(self):
        """
        Use this parameter to enable priority queuing on the specified virtual server.
        Default value: OFF
        """
        return self.options['pq']

    def set_sc(self, sc):
        """
        Use this parameter to enable SureConnect on the specified virtual server.
        Default value: OFF
        """
        self.options['sc'] = sc

    def get_sc(self):
        """
        Use this parameter to enable SureConnect on the specified virtual server.
        Default value: OFF
        """
        return self.options['sc']

    def set_rtspnat(self, rtspnat):
        """
        Use this parameter to enable natting for RTSP data connection.
        Default value: OFF
        """
        self.options['rtspnat'] = rtspnat

    def get_rtspnat(self):
        """
        Use this parameter to enable natting for RTSP data connection.
        Default value: OFF
        """
        return self.options['rtspnat']

    def set_m(self, m):
        """
        Use this parameter to specify the LB mode. If the value is specified as IP then the traffic
        is sent to the physical servers by changing the destination IP address to that of the physical server.
        If the value is MAC then the traffic is sent to the physical servers , by changing
        the destination MAC address to that of one of the physical servers, the destination IP is not changed.
        MAC mode is used mostly in Firewall Load Balancing scenario.
        Default value: IP
        """
        self.options['m'] = m

    def get_m(self):
        """
        Use this parameter to specify the LB mode. If the value is specified as IP then the traffic
        is sent to the physical servers by changing the destination IP address to that of the physical server.
        If the value is MAC then the traffic is sent to the physical servers , by changing
        the destination MAC address to that of one of the physical servers, the destination IP is not changed.
        MAC mode is used mostly in Firewall Load Balancing scenario.
        Default value: IP
        """
        return self.options['m']

    def set_tosid(self, tosid):
        """
        Use this parameter to specify the TOS ID of this vserver. Applicable only when the LB mode is TOS.
        Default value: 0
        Minimum value =  1
        Maximum value =  63
        """
        self.options['tosid'] = tosid

    def get_tosid(self):
        """
        Use this parameter to specify the TOS ID of this vserver. Applicable only when the LB mode is TOS.
        Default value: 0
        Minimum value =  1
        Maximum value =  63
        """
        return self.options['tosid']

    def set_datalength(self, datalength):
        """
        Use this parameter to specify the length of the token in bytes. Applicable to
        TCP virtual servers, when Token Load Balancing method is selected.
        The datalength should not be more than 24k.
        Default value: 0
        Minimum value =  0
        Maximum value =  100
        """
        self.options['datalength'] = datalength

    def get_datalength(self):
        """
        Use this parameter to specify the length of the token in bytes. Applicable to
        TCP virtual servers, when Token Load Balancing method is selected.
        The datalength should not be more than 24k.
        Default value: 0
        Minimum value =  0
        Maximum value =  100
        """
        return self.options['datalength']

    def set_dataoffset(self, dataoffset):
        """
        Use this parameter to specifies offset of the data to be taken as token. Applicable to
        the TCP type virtual servers, when Token load balancing method is used.
        Must be within the first 24k of the client TCP data.
        Default value: 0
        Minimum value =  0
        Maximum value =  25400
        """
        self.options['dataoffset'] = dataoffset

    def get_dataoffset(self):
        """
        Use this parameter to specifies offset of the data to be taken as token. Applicable to
        the TCP type virtual servers, when Token load balancing method is used.
        Must be within the first 24k of the client TCP data.
        Default value: 0
        Minimum value =  0
        Maximum value =  25400
        """
        return self.options['dataoffset']

    def set_sessionless(self, sessionless):
        """
        Use this parameter to enable sessionless load balancing.
        Default value: DISABLED
        """
        self.options['sessionless'] = sessionless

    def get_sessionless(self):
        """
        Use this parameter to enable sessionless load balancing.
        Default value: DISABLED
        """
        return self.options['sessionless']

    def set_state(self, state):
        """
        The state of the load balancing virtual server.
        Default value: ENABLED
        """
        self.options['state'] = state

    def get_state(self):
        """
        The state of the load balancing virtual server.
        Default value: ENABLED
        """
        return self.options['state']

    def set_connfailover(self, connfailover):
        """
        Specifies the connection failover mode of the virtual server.
        Default value: DISABLED
        """
        self.options['connfailover'] = connfailover

    def get_connfailover(self):
        """
        Specifies the connection failover mode of the virtual server.
        Default value: DISABLED
        """
        return self.options['connfailover']

    def set_redirurl(self, redirurl):
        """
        The URL where traffic is redirected if the virtual server in the system becomes unavailable.
        You can enter up to 127 characters as the URL argument.
        WARNING!        Make sure that the domain you specify in the URL does not match the domain specified
        in the -d domainName argument of the add cs policy CLI command. If the same domain is specified
        in both arguments, the request will be
        continuously redirected to the same unavailable virtual server in the system  -  then the user
        may not get the requested content.
        Default value: 0
        Minimum length =  1.
        """
        self.options['redirurl'] = redirurl

    def get_redirurl(self):
        """
        The URL where traffic is redirected if the virtual server in the system becomes unavailable.
        You can enter up to 127 characters as the URL argument.
        WARNING!        Make sure that the domain you specify in the URL does not match the domain specified
        in the -d domainName argument of the add cs policy CLI command. If the same domain is specified
        in both arguments, the request will be
        continuously redirected to the same unavailable virtual server in the system  -  then the user
        may not get the requested content.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['redirurl']

    def set_cacheable(self, cacheable):
        """
        Use this option to specify whether a virtual server, used for load balancing or content switching,
        routes requests to the cache redirection virtual server before sending it to the configured servers.
        Default value: NO
        """
        self.options['cacheable'] = cacheable

    def get_cacheable(self):
        """
        Use this option to specify whether a virtual server, used for load balancing or content switching,
        routes requests to the cache redirection virtual server before sending it to the configured servers.
        Default value: NO
        """
        return self.options['cacheable']

    def set_clttimeout(self, clttimeout):
        """
        The timeout value in seconds for idle client connection.
        Default value: 0
        Minimum value =  0
        Maximum value =  31536000
        """
        self.options['clttimeout'] = clttimeout

    def get_clttimeout(self):
        """
        The timeout value in seconds for idle client connection.
        Default value: 0
        Minimum value =  0
        Maximum value =  31536000
        """
        return self.options['clttimeout']

    def set_somethod(self, somethod):
        """
        The spillover factor based on which the traffic will be given to the backupvserver
        once the main virtual server reaches the spillover threshold.
        Default value: 0
        """
        self.options['somethod'] = somethod

    def get_somethod(self):
        """
        The spillover factor based on which the traffic will be given to the backupvserver
        once the main virtual server reaches the spillover threshold.
        Default value: 0
        """
        return self.options['somethod']

    def set_sopersistence(self, sopersistence):
        """
        The state of the spillover persistence.
        Default value: DISABLED
        """
        self.options['sopersistence'] = sopersistence

    def get_sopersistence(self):
        """
        The state of the spillover persistence.
        Default value: DISABLED
        """
        return self.options['sopersistence']

    def set_sopersistencetimeout(self, sopersistencetimeout):
        """
        The spillover persistency entry timeout.
        Default value: 2
        Minimum value =  2
        Maximum value =  1440
        """
        self.options['sopersistencetimeout'] = sopersistencetimeout

    def get_sopersistencetimeout(self):
        """
        The spillover persistency entry timeout.
        Default value: 2
        Minimum value =  2
        Maximum value =  1440
        """
        return self.options['sopersistencetimeout']

    def set_sothreshold(self, sothreshold):
        """
        In case of CONNECTION (or) DYNAMICCONNECTION type spillover method this value is treated
        as Maximum number of connections an lb vserver will handle before spillover takes place.
        In case of BANDWIDTH type spillover method this value is treated as the amount
        of incoming and outgoing traffic (in Kbps) a Vserver will handle before spillover takes place.
        In case of HEALTH type spillover method if the percentage of active services
        (by weight) becomes lower than this value, spillover takes place .
        Default value: 0
        Minimum value =  1
        Maximum value =  4294967287
        """
        self.options['sothreshold'] = sothreshold

    def get_sothreshold(self):
        """
        In case of CONNECTION (or) DYNAMICCONNECTION type spillover method this value is treated
        as Maximum number of connections an lb vserver will handle before spillover takes place.
        In case of BANDWIDTH type spillover method this value is treated as the amount
        of incoming and outgoing traffic (in Kbps) a Vserver will handle before spillover takes place.
        In case of HEALTH type spillover method if the percentage of active services
        (by weight) becomes lower than this value, spillover takes place .
        Default value: 0
        Minimum value =  1
        Maximum value =  4294967287
        """
        return self.options['sothreshold']

    def set_redirectportrewrite(self, redirectportrewrite):
        """
        The state of port rewrite while performing HTTP redirect.
        Default value: DISABLED
        """
        self.options['redirectportrewrite'] = redirectportrewrite

    def get_redirectportrewrite(self):
        """
        The state of port rewrite while performing HTTP redirect.
        Default value: DISABLED
        """
        return self.options['redirectportrewrite']

    def set_downstateflush(self, downstateflush):
        """
        Perform delayed clean up of connections on this vserver.
        Default value: ENABLED
        """
        self.options['downstateflush'] = downstateflush

    def get_downstateflush(self):
        """
        Perform delayed clean up of connections on this vserver.
        Default value: ENABLED
        """
        return self.options['downstateflush']

    def set_backupvserver(self, backupvserver):
        """
        The Backup Virtual Server.
        Default value: 0
        Minimum length =  1.
        """
        self.options['backupvserver'] = backupvserver

    def get_backupvserver(self):
        """
        The Backup Virtual Server.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['backupvserver']

    def set_disableprimaryondown(self, disableprimaryondown):
        """
        When this argument is enabled, traffic will continue reaching backup vservers
        even after primary comes UP from DOWN state.
        Default value: DISABLED
        """
        self.options['disableprimaryondown'] = disableprimaryondown

    def get_disableprimaryondown(self):
        """
        When this argument is enabled, traffic will continue reaching backup vservers
        even after primary comes UP from DOWN state.
        Default value: DISABLED
        """
        return self.options['disableprimaryondown']

    def set_insertvserveripport(self, insertvserveripport):
        """
        The virtual IP and port header insertion option for the vserver.
               VIPADDR      - Header contains the vserver's IP address and port number without any translation.
               OFF             - The virtual IP and port header insertion option is disabled.
               V6TOV4MAPPING   - Header contains the mapped IPv4 address corresponding to the IPv6 address
               of the vserver and the port number. An IPv6 address can be mapped to
               a user-specified IPv4 address using the set ns ip6 command.
        Default value: OFF
        """
        self.options['insertvserveripport'] = insertvserveripport

    def get_insertvserveripport(self):
        """
        The virtual IP and port header insertion option for the vserver.
               VIPADDR      - Header contains the vserver's IP address and port number without any translation.
               OFF             - The virtual IP and port header insertion option is disabled.
               V6TOV4MAPPING   - Header contains the mapped IPv4 address corresponding to the IPv6 address
               of the vserver and the port number. An IPv6 address can be mapped to
        a user-specified IPv4 address using the set ns ip6 command.
        Default value: OFF
        """
        return self.options['insertvserveripport']

    def set_vipheader(self, vipheader):
        """
        The name of virtual IP and port header.
        Default value: 0
        Minimum length =  1.
        """
        self.options['vipheader'] = vipheader

    def get_vipheader(self):
        """
        The name of virtual IP and port header.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['vipheader']

    def set_authenticationhost(self, authenticationhost):
        """
        FQDN of authentication vserver.
        Default value: 0
        Minimum length =  3.
        Maximum length =  252.
        """
        self.options['authenticationhost'] = authenticationhost

    def get_authenticationhost(self):
        """
        FQDN of authentication vserver.
        Default value: 0
        Minimum length =  3.
        Maximum length =  252.
        """
        return self.options['authenticationhost']

    def set_authentication(self, authentication):
        """
        This option toggles on or off the application of authentication of incoming users to the vserver.
        Default value: OFF
        """
        self.options['authentication'] = authentication

    def get_authentication(self):
        """
        This option toggles on or off the application of authentication of incoming users to the vserver.
        Default value: OFF
        """
        return self.options['authentication']

    def set_authn401(self, authn401):
        """
        This option toggles on or off the HTTP 401 response based authentication.
        Default value: OFF
        """
        self.options['authn401'] = authn401

    def get_authn401(self):
        """
        This option toggles on or off the HTTP 401 response based authentication.
        Default value: OFF
        """
        return self.options['authn401']

    def set_authnvsname(self, authnvsname):
        """
        Name of authentication vserver.
        Default value: 0
        Minimum length =  1.
        Maximum length =  252.
        """
        self.options['authnvsname'] = authnvsname

    def get_authnvsname(self):
        """
        Name of authentication vserver.
        Default value: 0
        Minimum length =  1.
        Maximum length =  252.
        """
        return self.options['authnvsname']

    def set_push(self, push):
        """
        Process traffic on bound Push vserver.
        Default value: DISABLED
        """
        self.options['push'] = push

    def get_push(self):
        """
        Process traffic on bound Push vserver.
        Default value: DISABLED
        """
        return self.options['push']

    def set_pushvserver(self, pushvserver):
        """
        The lb vserver of type PUSH/SSL_PUSH to which server pushes
        the updates received on the client facing non-push lb vserver.
        Default value: 0
        Minimum length =  1.
        """
        self.options['pushvserver'] = pushvserver

    def get_pushvserver(self):
        """
        The lb vserver of type PUSH/SSL_PUSH to which server pushes the updates received
        on the client facing non-push lb vserver.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['pushvserver']

    def set_pushlabel(self, pushlabel):
        """
        Use this parameter to specify the expression to extract the label in response from server.
        The string can be either a named expression (configured using add policy expression command) or
        else it can be an in-line expression with a maximum of 63 characters.
        Default value: "none"
        """
        self.options['pushlabel'] = pushlabel

    def get_pushlabel(self):
        """
        Use this parameter to specify the expression to extract the label in response from server.
        The string can be either a named expression (configured using add policy expression command) or
        else it can be an in-line expression with a maximum of 63 characters.
        Default value: "none"
        """
        return self.options['pushlabel']

    def set_pushmulticlients(self, pushmulticlients):
        """
        Specify if multiple web 2.0 connections from the same client can connect
        to this vserver and expect updates.
        Default value: NO
        """
        self.options['pushmulticlients'] = pushmulticlients

    def get_pushmulticlients(self):
        """
        Specify if multiple web 2.0 connections from the same client can connect
        to this vserver and expect updates.
        Default value: NO
        """
        return self.options['pushmulticlients']

    def set_tcpprofilename(self, tcpprofilename):
        """
        The name of the TCP profile.
        Default value: 0
        Minimum length =  1.
        Maximum length =  127.
        """
        self.options['tcpprofilename'] = tcpprofilename

    def get_tcpprofilename(self):
        """
        The name of the TCP profile.
        Default value: 0
        Minimum length =  1.
        Maximum length =  127.
        """
        return self.options['tcpprofilename']

    def set_httpprofilename(self, httpprofilename):
        """
        Name of the HTTP profile.
        Default value: 0
        Minimum length =  1.
        Maximum length =  127.
        """
        self.options['httpprofilename'] = httpprofilename

    def get_httpprofilename(self):
        """
        Name of the HTTP profile.
        Default value: 0
        Minimum length =  1.
        Maximum length =  127.
        """
        return self.options['httpprofilename']

    def set_comment(self, comment):
        """
        Comments associated with this virtual server.
        Default value: 0
        """
        self.options['comment'] = comment

    def get_comment(self):
        """
        Comments associated with this virtual server.
        Default value: 0
        """
        return self.options['comment']

    def set_weight(self, weight):
        """
        The weight for the specified service.
        Default value: 0
        Minimum value =  1
        Maximum value =  100
        """
        self.options['weight'] = weight

    def get_weight(self):
        """
        The weight for the specified service.
        Default value: 0
        Minimum value =  1
        Maximum value =  100
        """
        return self.options['weight']

    def set_servicename(self, servicename):
        """
        The service name bound to the selected load balancing virtual server.
        Default value: 0
        Minimum length =  1.
        """
        self.options['servicename'] = servicename

    def get_servicename(self):
        """
        The service name bound to the selected load balancing virtual server.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['servicename']

    def set_redirurlflags(self, redirurlflags):
        """
        The redirect URL to be unset.
        """
        self.options['redirurlflags'] = redirurlflags

    def get_redirurlflags(self):
        """
        The redirect URL to be unset.
        """
        return self.options['redirurlflags']

    def set_newname(self, newname):
        """
        The new name of the virtual server.
        Default value: 0
        Minimum length =  1.
        """
        self.options['newname'] = newname

    def get_newname(self):
        """
        The new name of the virtual server.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['newname']

    def get_value(self):
        """
        SSL status.
        Default value: 0
        """
        return self.options['value']

    def get_ipmapping(self):
        """
        The permanent mapping for the V6 Address.
        Default value: 0
        """
        return self.options['ipmapping']

    def get_type(self):
        """
        Type of LB vserver.
        Default value: 0
        """
        return self.options['type']

    def get_curstate(self):
        """
        Current LB vserver state.
        Default value: 0
        """
        return self.options['curstate']

    def get_effectivestate(self):
        """
        Effective state of the LB vserver , based on the state of backup vservers.
        Default value: 0
        """
        return self.options['effectivestate']

    def get_status(self):
        """
        Current status of the lb vserver. During the initial phase if the configured lb method is not
        round robin , the vserver will adopt round robin to distribute
        traffic for a predefined number of requests.
        Default value: 0
        """
        return self.options['status']

    def get_lbrrreason(self):
        """
        Reason why a vserver is in RR. The following are the reasons:
        1  - MEP is DOWN (GSLB)
        2  - LB method has changed
        3  - Bound service's state changed to UP
        4  - A new service is bound
        5  - Startup RR factor has changed
        6  - LB feature is enabled
        7  - Load monitor is not active on a service
        8  - Vserver is Enabled
        9  - SSL feature is Enabled
        10 - All bound services have reached threshold. Using effective state to load balance (GSLB)
        11 - Primary state of bound services are not UP. Using effective state to load balance (GSLB)
        12 - No LB decision can be made as all bound services have either reached threshold or are not UP (GSLB)
        13 - All load monitors are active
        .
        Default value: 0
        """
        return self.options['lbrrreason']

    def get_redirect(self):
        """
        Cache redirect type.
        Default value: 0
        """
        return self.options['redirect']

    def get_precedence(self):
        """
        Precedence.
        Default value: 0
        """
        return self.options['precedence']

    def get_homepage(self):
        """
        Home page.
        Default value: 0
        """
        return self.options['homepage']

    def get_dnsvservername(self):
        """
        DNS vserver name.
        Default value: 0
        """
        return self.options['dnsvservername']

    def get_domain(self):
        """
        Domain.
        Default value: 0
        """
        return self.options['domain']

    def get_cachevserver(self):
        """
        Cache virtual server.
        Default value: 0
        """
        return self.options['cachevserver']

    def get_health(self):
        """
        Health of vserver based on percentage of weights of active svcs/all svcs.
        This does not consider administratively disabled svcs.
        Default value: 0
        """
        return self.options['health']

    def get_gotopriorityexpression(self):
        """
        Expression specifying the priority of the next policy which will get evaluated
        if the current policy rule evaluates to TRUE.
        Default value: 0
        """
        return self.options['gotopriorityexpression']

    def get_ruletype(self):
        """
        Rule type.
        Default value: 0
        """
        return self.options['ruletype']

    def get_groupname(self):
        """
        LB group to which the lb vserver is to be bound.
        Default value: 0
        """
        return self.options['groupname']

    def get_cookiedomain(self):
        """
        Domain name to be used in the set cookie header in case of cookie persistence.
        Default value: 0
        """
        return self.options['cookiedomain']

    def get_map(self):
        """
        Map.
        Default value: 0
        """
        return self.options['map']

    def get_gt2gb(self):
        """
        Allow for greater than 2 GB transactions on this vserver.
        Default value: 0
        """
        return self.options['gt2gb']

    def get_thresholdvalue(self):
        """
        Tells whether threshold exceeded for this service participating in CUSTOMLB.
        Default value: 0
        """
        return self.options['thresholdvalue']

    def get_bindpoint(self):
        """
        The bindpoint to which the policy is bound.
        Default value: 0
        """
        return self.options['bindpoint']

    def get_invoke(self):
        """
        Invoke flag.
        Default value: 0
        """
        return self.options['invoke']

    def get_labeltype(self):
        """
        The invocation type.
        Default value: 0
        """
        return self.options['labeltype']

    def get_labelname(self):
        """
        Name of the label invoked.
        Default value: 0
        """
        return self.options['labelname']

    def get_version(self):
        """
        Cookie version.
        Default value: 0
        """
        return self.options['version']

    def get_totalservices(self):
        """
        Total number of services bound to the vserver.
        Default value: 0
        """
        return self.options['totalservices']

    def get_activeservices(self):
        """
        Total number of active services bound to the vserver.
        Default value: 0
        """
        return self.options['activeservices']

    def get_statechangetimeseconds(self):
        """
        Time when last state change happened. Seconds part.
        Default value: 0
        """
        return self.options['statechangetimeseconds']

    def get_statechangetimemsec(self):
        """
        Time at which last state change happened. Milliseconds part.
        Default value: 0
        """
        return self.options['statechangetimemsec']

    def get_tickssincelaststatechange(self):
        """
        Time in 10 millisecond ticks since the last state change.
        Default value: 0
        """
        return self.options['tickssincelaststatechange']

    # Operations methods
    @staticmethod
    def disable(nitro, lbvserver):
        """
        Use this API to disable lbvserver.
        """
        __lbvserver = NSLBVServer()
        __lbvserver.set_name(lbvserver.get_name())
        return __lbvserver.perform_operation(nitro, "disable")

    @staticmethod
    def enable(nitro, lbvserver):
        """
        Use this API to enable lbvserver.
        """
        __lbvserver = NSLBVServer()
        __lbvserver.set_name(lbvserver.get_name())
        return __lbvserver.perform_operation(nitro, "enable")

    @staticmethod
    def rename(nitro, lbvserver):
        """
        Use this API to rename lbvserver.
        """
        __lbvserver = NSLBVServer()
        __lbvserver.set_name(lbvserver.get_name())
        __lbvserver.set_newname(lbvserver.get_newname())
        return __lbvserver.perform_operation(nitro, "rename")

    @staticmethod
    def get(nitro, lbvserver):
        """
        Use this API to fetch lbvserver resource of given name.
        """
        __lbvserver = NSLBVServer()
        __lbvserver.set_name(lbvserver.get_name())
        __lbvserver.get_resource(nitro)
        return __lbvserver

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured lbvserver resources.
        """
        __url = nitro.get_url() + NSLBVServer.get_resourcetype()
        __json_lbvservers = nitro.get(__url).get_response_field(NSLBVServer.get_resourcetype())
        __lbvservers = []
        for json_lbvserver in __json_lbvservers:
            __lbvservers.append(NSLBVServer(json_lbvserver))
        return __lbvservers

    @staticmethod
    def add(nitro, lbvserver):
        """
        Use this API to add lbvserver.
        """
        __lbvserver = NSLBVServer()
        __lbvserver.set_name(lbvserver.get_name())
        __lbvserver.set_servicetype(lbvserver.get_servicetype())
        __lbvserver.set_ipv46(lbvserver.get_ipv46())
        __lbvserver.set_ippattern(lbvserver.get_ippattern())
        __lbvserver.set_ipmask(lbvserver.get_ipmask())
        __lbvserver.set_port(lbvserver.get_port())
        __lbvserver.set_range(lbvserver.get_range())
        __lbvserver.set_persistencetype(lbvserver.get_persistencetype())
        __lbvserver.set_timeout(lbvserver.get_timeout())
        __lbvserver.set_persistencebackup(lbvserver.get_persistencebackup())
        __lbvserver.set_backuppersistencetimeout(lbvserver.get_backuppersistencetimeout())
        __lbvserver.set_lbmethod(lbvserver.get_lbmethod())
        __lbvserver.set_hashlength(lbvserver.get_hashlength())
        __lbvserver.set_netmask(lbvserver.get_netmask())
        __lbvserver.set_v6netmasklen(lbvserver.get_v6netmasklen())
        __lbvserver.set_rule(lbvserver.get_rule())
        __lbvserver.set_listenpolicy(lbvserver.get_listenpolicy())
        __lbvserver.set_listenpriority(lbvserver.get_listenpriority())
        __lbvserver.set_resrule(lbvserver.get_resrule())
        __lbvserver.set_persistmask(lbvserver.get_persistmask())
        __lbvserver.set_v6persistmasklen(lbvserver.get_v6persistmasklen())
        __lbvserver.set_pq(lbvserver.get_pq())
        __lbvserver.set_sc(lbvserver.get_sc())
        __lbvserver.set_rtspnat(lbvserver.get_rtspnat())
        __lbvserver.set_m(lbvserver.get_m())
        __lbvserver.set_tosid(lbvserver.get_tosid())
        __lbvserver.set_datalength(lbvserver.get_datalength())
        __lbvserver.set_dataoffset(lbvserver.get_dataoffset())
        __lbvserver.set_sessionless(lbvserver.get_sessionless())
        __lbvserver.set_state(lbvserver.get_state())
        __lbvserver.set_connfailover(lbvserver.get_connfailover())
        __lbvserver.set_redirurl(lbvserver.get_redirurl())
        __lbvserver.set_cacheable(lbvserver.get_cacheable())
        __lbvserver.set_clttimeout(lbvserver.get_clttimeout())
        __lbvserver.set_somethod(lbvserver.get_somethod())
        __lbvserver.set_sopersistence(lbvserver.get_sopersistence())
        __lbvserver.set_sopersistencetimeout(lbvserver.get_sopersistencetimeout())
        __lbvserver.set_sothreshold(lbvserver.get_sothreshold())
        __lbvserver.set_redirectportrewrite(lbvserver.get_redirectportrewrite())
        __lbvserver.set_downstateflush(lbvserver.get_downstateflush())
        __lbvserver.set_backupvserver(lbvserver.get_backupvserver())
        __lbvserver.set_disableprimaryondown(lbvserver.get_disableprimaryondown())
        __lbvserver.set_insertvserveripport(lbvserver.get_insertvserveripport())
        __lbvserver.set_vipheader(lbvserver.get_vipheader())
        __lbvserver.set_authenticationhost(lbvserver.get_authenticationhost())
        __lbvserver.set_authentication(lbvserver.get_authentication())
        __lbvserver.set_authn401(lbvserver.get_authn401())
        __lbvserver.set_authnvsname(lbvserver.get_authnvsname())
        __lbvserver.set_push(lbvserver.get_push())
        __lbvserver.set_pushvserver(lbvserver.get_pushvserver())
        __lbvserver.set_pushlabel(lbvserver.get_pushlabel())
        __lbvserver.set_pushmulticlients(lbvserver.get_pushmulticlients())
        __lbvserver.set_tcpprofilename(lbvserver.get_tcpprofilename())
        __lbvserver.set_httpprofilename(lbvserver.get_httpprofilename())
        __lbvserver.set_comment(lbvserver.get_comment())
        return __lbvserver.add_resource(nitro)

    @staticmethod
    def delete(nitro, lbvserver):
        """
        Use this API to delete lbvserver of a given name.
        """
        __lbvserver = NSLBVServer()
        __lbvserver.set_name(lbvserver.get_name())
        nsresponse = __lbvserver.delete_resource(nitro)
        return nsresponse

    @staticmethod
    def update(nitro, lbvserver):
        """
        Use this API to update lbvserver of a given name.
        """
        __lbvserver = NSLBVServer()
        __lbvserver.set_name(lbvserver.get_name())
        __lbvserver.set_ipv46(lbvserver.get_ipv46())
        __lbvserver.set_ippattern(lbvserver.get_ippattern())
        __lbvserver.set_ipmask(lbvserver.get_ipmask())
        __lbvserver.set_weight(lbvserver.get_weight())
        __lbvserver.set_servicename(lbvserver.get_servicename())
        __lbvserver.set_persistencetype(lbvserver.get_persistencetype())
        __lbvserver.set_timeout(lbvserver.get_timeout())
        __lbvserver.set_persistencebackup(lbvserver.get_persistencebackup())
        __lbvserver.set_backuppersistencetimeout(lbvserver.get_backuppersistencetimeout())
        __lbvserver.set_lbmethod(lbvserver.get_lbmethod())
        __lbvserver.set_hashlength(lbvserver.get_hashlength())
        __lbvserver.set_netmask(lbvserver.get_netmask())
        __lbvserver.set_v6netmasklen(lbvserver.get_v6netmasklen())
        __lbvserver.set_rule(lbvserver.get_rule())
        __lbvserver.set_resrule(lbvserver.get_resrule())
        __lbvserver.set_persistmask(lbvserver.get_persistmask())
        __lbvserver.set_v6persistmasklen(lbvserver.get_v6persistmasklen())
        __lbvserver.set_pq(lbvserver.get_pq())
        __lbvserver.set_sc(lbvserver.get_sc())
        __lbvserver.set_rtspnat(lbvserver.get_rtspnat())
        __lbvserver.set_m(lbvserver.get_m())
        __lbvserver.set_tosid(lbvserver.get_tosid())
        __lbvserver.set_datalength(lbvserver.get_datalength())
        __lbvserver.set_dataoffset(lbvserver.get_dataoffset())
        __lbvserver.set_sessionless(lbvserver.get_sessionless())
        __lbvserver.set_connfailover(lbvserver.get_connfailover())
        __lbvserver.set_backupvserver(lbvserver.get_backupvserver())
        __lbvserver.set_redirurl(lbvserver.get_redirurl())
        __lbvserver.set_cacheable(lbvserver.get_cacheable())
        __lbvserver.set_clttimeout(lbvserver.get_clttimeout())
        __lbvserver.set_somethod(lbvserver.get_somethod())
        __lbvserver.set_sothreshold(lbvserver.get_sothreshold())
        __lbvserver.set_sopersistence(lbvserver.get_sopersistence())
        __lbvserver.set_sopersistencetimeout(lbvserver.get_sopersistencetimeout())
        __lbvserver.set_redirectportrewrite(lbvserver.get_redirectportrewrite())
        __lbvserver.set_downstateflush(lbvserver.get_downstateflush())
        __lbvserver.set_insertvserveripport(lbvserver.get_insertvserveripport())
        __lbvserver.set_vipheader(lbvserver.get_vipheader())
        __lbvserver.set_disableprimaryondown(lbvserver.get_disableprimaryondown())
        __lbvserver.set_authenticationhost(lbvserver.get_authenticationhost())
        __lbvserver.set_authentication(lbvserver.get_authentication())
        __lbvserver.set_authn401(lbvserver.get_authn401())
        __lbvserver.set_authnvsname(lbvserver.get_authnvsname())
        __lbvserver.set_push(lbvserver.get_push())
        __lbvserver.set_pushvserver(lbvserver.get_pushvserver())
        __lbvserver.set_pushlabel(lbvserver.get_pushlabel())
        __lbvserver.set_pushmulticlients(lbvserver.get_pushmulticlients())
        __lbvserver.set_listenpolicy(lbvserver.get_listenpolicy())
        __lbvserver.set_listenpriority(lbvserver.get_listenpriority())
        __lbvserver.set_tcpprofilename(lbvserver.get_tcpprofilename())
        __lbvserver.set_httpprofilename(lbvserver.get_httpprofilename())
        __lbvserver.set_comment(lbvserver.get_comment())
        return __lbvserver.update_resource(nitro)



from nsbaseresource import NSBaseResource

__author__ = 'ndenev@gmail.com'


class SNMPManager(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(SNMPManager, self).__init__()

        self.options = {'ipaddress': '',
                        'netmask': '',
                        'domainresolveretry': '',
                        # ReadOnly
                        'ip': '',
                        'domain': ''}

        self.resourcetype = SNMPManager.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "snmpmanager"

    def set_ipaddress(self, ipaddress):
        self.options['ipaddress'] = ipaddress

    def get_ipaddress(self):
        return self.options['ipaddress']

    def set_netmask(self, netmask):
        self.options['netmask'] = netmask

    def get_netmask(self):
        return self.options['netmask']

    def set_domainresolveretry(self, domainresolveretry):
        self.options['domainresolveretry'] = domainresolveretry

    def get_domainresolveretry(self):
        return self.options['domainresolveretry']

    def get_ip(self):
        return self.options['ip']

    def get_domain(self):
        return self.options['domain']

    @staticmethod
    def add(nitro, snmpmanager):
        __snmpmanager = SNMPManager()
        __snmpmanager.set_ipaddress(snmpmanager.get_ipaddress())
        __snmpmanager.set_netmask(snmpmanager.get_netmask())
        __snmpmanager.set_domainresolveretry(snmpmanager.get_domainresolveretry())
        return __snmpmanager.add_resource(nitro)

    @staticmethod
    def delete(nitro, snmpmanager):
        __snmpmanager = SNMPManager()
        __snmpmanager.set_ipaddress(snmpmanager.get_ipaddress())
        return __snmpmanager.delete_resource(nitro)

    @staticmethod
    def update(nitro, snmpmanager):
        __snmpmanager = SNMPManager()
        __snmpmanager.set_ipaddress(snmpmanager.get_ipaddress())
        __snmpmanager.set_netmask(snmpmanager.get_netmask())
        __snmpmanager.set_domainresolveretry(snmpmanager.get_domainresolveretry())
        return __snmpmanager.update_resource(nitro)

    @staticmethod
    def get(nitro, snmpmanager):
        __snmpmanager = SNMPManager()
        __snmpmanager.set_ipaddress(snmpmanager.get_ipaddress())
        __snmpmanager.get_resource(nitro, object_name=__snmpmanager.get_ipaddress())
        return __snmpmanager

    @staticmethod
    def get_all(nitro):
        __url = nitro.get_url() + SNMPManager.get_resourcetype()
        __json_snmpmanagers = nitro.get(__url).get_response_field(SNMPManager.get_resourcetype())
        __snmpmanagers = []
        for json_snmpmanager in __json_snmpmanagers:
            __snmpmanagers.append(SNMPManager(json_snmpmanager))
        return __snmpmanagers




from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSServer(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSServer, self).__init__()

        self.options = {
            # Read-write values
            'name': '',
            'ipaddress': '',
            'domain': '',
            'translationip': '',
            'translationmask': '',
            'domainresolveretry': '',
            'state': '',
            'ipv6address': '',
            'comment': '',
            'domainresolvenow': '',
            'delay': '',
            'graceful': '',
            'newname': ''
        }

        self.resourcetype = NSServer.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options:
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "server"

    def set_name(self, name):
        """
        The server's name.
        Default value: 0
        Minimum length =  1.
        """
        self.options['name'] = name

    def get_name(self):
        """
        The server's name.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['name']

    def set_ipaddress(self, ipaddress):
        """
        The IP address of the server.
        Default value: 0
        """
        self.options['ipaddress'] = ipaddress

    def get_ipaddress(self):
        """
        The IP address of the server.
        Default value: 0
        """
        return self.options['ipaddress']

    def set_domain(self, domain):
        """
        The domain name of the server for which a service needs to be added. If an IP Address has been
        specified, the domain name does not need to be specified.

        Default value: 0
        Minimum length =  1.
        """
        self.options['domain'] = domain

    def get_domain(self):
        """
        The domain name of the server for which a service needs to be added. If an IP Address has been
        specified, the domain name does not need to be specified.

        Default value: 0
        Minimum length =  1.
        """
        return self.options['domain']

    def set_translationip(self, translationip):
        """
        The IP address used for translating dns obtained ip.
        Default value: 0
        """
        self.options['translationip'] = translationip

    def get_translationip(self):
        """
        The IP address used for translating dns obtained ip.
        Default value: 0
        """
        return self.options['translationip']

    def set_translationmask(self, translationmask):
        """
        The netmask of the translation ip.
        Default value: 0
        """
        self.options['translationmask'] = translationmask

    def get_translationmask(self):
        """
        The netmask of the translation ip.
        Default value: 0
        """
        return self.options['translationmask']

    def set_domainresolveretry(self, domainresolveretry):
        """
        The duration in seconds for which NetScaler system waits to send the next dns query to resolve the
        domain name, in case the last query failed. If last query succeeds, the netscaler system waits for TTL
        time in the response.
        Default value: 5
        Minimum value =  5
        Maximum value =  20940
        """
        self.options['domainresolveretry'] = domainresolveretry

    def get_domainresolveretry(self):
        """
        The duration in seconds for which NetScaler system waits to send the next dns query to resolve the
        domain name, in case the last query failed. If last query succeeds, the netscaler system waits for TTL
        time in the response.
        Default value: 5
        Minimum value =  5
        Maximum value =  20940
        """
        return self.options['domainresolveretry']

    def set_state(self, state):
        """
        The initial state of the server.
        Default value: ENABLED
        """
        self.options['state'] = state

    def get_state(self):
        """
        The initial state of the server.
        Default value: ENABLED
        """
        return self.options['state']

    def set_ipv6address(self, ipv6address):
        """
        Defines whether server is of type ipv6 or not for DBS services.
        Default value: NO
        """
        self.options['ipv6address'] = ipv6address

    def get_ipv6address(self):
        """
        Defines whether server is of type ipv6 or not for DBS services.
        Default value: NO
        """
        return self.options['ipv6address']

    def set_comment(self, comment):
        """
        Comments associated with this server.
        Default value: 0
        """
        self.options['comment'] = comment

    def get_comment(self):
        """
        Comments associated with this server.
        Default value: 0
        """
        return self.options['comment']

    def set_domainresolvenow(self, domainresolvenow):
        """
        Restart the probe for this domain based server, immediately.
        Default value: 0
        """
        self.options['domainresolvenow'] = domainresolvenow

    def get_domainresolvenow(self):
        """
        Restart the probe for this domain based server, immediately.
        Default value: 0
        """
        return self.options['domainresolvenow']

    def set_delay(self, delay):
        """
        The time in seconds after which all services in this server are brought down.
        Default value: 0
        """
        self.options['delay'] = delay

    def get_delay(self):
        """
        The time in seconds after which all services in this server are brought down.
        Default value: 0
        """
        return self.options['delay']

    def set_graceful(self, graceful):
        """
        Indicates graceful shutdown of the service. System will wait for all outstanding
        connections to this service to be closed before disabling the service.
        Default value: NO
        """
        self.options['graceful'] = graceful

    def get_graceful(self):
        """
        Indicates graceful shutdown of the service. System will wait for all outstanding
        connections to this service to be closed before disabling the service.
        Default value: NO
        """
        return self.options['graceful']

    def set_newname(self, newname):
        """
        The new name of the server.
        Default value: 0
        Minimum length =  1.
        """
        self.options['newname'] = newname

    def get_newname(self):
        """
        The new name of the server.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['newname']

    # Operations methods
    @staticmethod
    def disable(nitro, server):
        """
        Use this API to disable server.
        """
        __server = NSServer()
        __server.set_name(server.get_name())
        __server.set_delay(server.get_delay())
        __server.set_graceful(server.get_graceful())
        return __server.perform_operation(nitro, "disable")

    @staticmethod
    def enable(nitro, server):
        """
        Use this API to enable server.
        """
        __server = NSServer()
        __server.set_name(server.get_name())
        return __server.perform_operation(nitro, "enable")

    @staticmethod
    def rename(nitro, server):
        """
        Use this API to rename server.
        """
        __server = NSServer()
        __server.set_name(server.get_name())
        __server.set_newname(server.get_newname())
        return __server.perform_operation(nitro, "rename")

    @staticmethod
    def get(nitro, server):
        """
        Use this API to fetch server resource of given name.
        """
        __server = NSServer()
        __server.set_name(server.get_name())
        __server.get_resource(nitro)
        return __server

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured server resources.
        """
        __url = nitro.get_url() + NSServer.get_resourcetype()
        __json_servers = nitro.get(__url).get_response_field(NSServer.get_resourcetype())
        __servers = []
        for json_server in __json_servers:
            __servers.append(NSServer(json_server))
        return __servers

    @staticmethod
    def add(nitro, server):
        """
        Use this API to add server.
        """
        __server = NSServer()
        __server.set_name(server.get_name())
        __server.set_ipaddress(server.get_ipaddress())
        __server.set_domain(server.get_domain())
        __server.set_translationip(server.get_translationip())
        __server.set_translationmask(server.get_translationmask())
        __server.set_domainresolveretry(server.get_domainresolveretry())
        __server.set_state(server.get_state())
        __server.set_ipv6address(server.get_ipv6address())
        __server.set_comment(server.get_comment())
        return __server.add_resource(nitro)

    @staticmethod
    def delete(nitro, server):
        """
        Use this API to delete server of a given name.
        """
        __server = NSServer()
        __server.set_name(server.get_name())
        nsresponse = __server.delete_resource(nitro)
        return nsresponse

    @staticmethod
    def update(nitro, server):
        """
        Use this API to update a server of a given name.
        """
        __server = NSServer()
        __server.set_name(server.get_name())
        __server.set_ipaddress(server.get_ipaddress())
        __server.set_domainresolveretry(server.get_domainresolveretry())
        __server.set_translationip(server.get_translationip())
        __server.set_translationmask(server.get_translationmask())
        __server.set_domainresolvenow(server.get_domainresolvenow())
        __server.set_comment(server.get_comment())
        return __server.update_resource(nitro)



from nsbaseresource import NSBaseResource
__author__ = 'ndenev'


class NSPbrs(NSBaseResource):

    def __init__(self):

        super(NSPbrs, self).__init__()

        self.options = {}
        self.resourcetype = NSPbrs.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "nspbrs"

    @staticmethod
    def renumber(nitro):
        """
        Use this API to renumber nspbrs.
        """
        __nsprbs = NSPbrs()
        return __nsprbs.perform_operation(nitro, "renumber")

    @staticmethod
    def clear(nitro):
        """
        Use this API to clear nspbrs.
        """
        __nspbrs = NSPbrs()
        return __nspbrs.perform_operation(nitro, "clear")

    @staticmethod
    def apply(nitro):
        """
        Use this API to apply nspbrs.
        """
        __nspbrs = NSPbrs()
        return __nspbrs.perform_operation(nitro, "apply")



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSRewritePolicy(NSBaseResource):

    # Configuration for content-switching policy resource.

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSRewritePolicy, self).__init__()
        self.options = {
            'name': '',
            'rule': '',
            'action': '',
            'undefaction': '',
            'comment': '',
            'logaction': '',
            'newname': '',
            'hits': '',
            'undefhits': '',
            'description': '',
        }

        self.resourcetype = NSRewritePolicy.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "rewritepolicy"

    def set_name(self, name):
        """
        Name of the rewrite policy.

        Default value: 0
        """
        self.options['name'] = name

    def get_name(self):
        """
        Name of the rewrite policy.

        Default value: 0
        """
        return self.options['name']

    def set_rule(self, rule):
        """
        Expression to be used by rewrite policy. It has to be a boolean PI rule expression.

        Default value: 0
        """
        self.options['rule'] = rule

    def get_rule(self):
        """
        Expression to be used by rewrite policy. It has to be a boolean PI rule expression.

        Default value: 0
        """
        return self.options['rule']

    def set_action(self, action):
        """
        Rewrite action to be used by the policy.

        Default value: 0
        """
        self.options['action'] = action

    def get_action(self):
        """
        Rewrite action to be used by the policy.

        Default value: 0
        """
        return self.options['action']

    def set_undefaction(self, undefaction):
        """
        A rewrite action, to be used by the policy when the rule evaluation turns out to be undefined.
        The undef action can be NOREWRITE, RESET or DROP.

        Default value: 0
        """
        self.options['undefaction'] = undefaction

    def get_undefaction(self):
        """
        A rewrite action, to be used by the policy when the rule evaluation turns out to be undefined.
        The undef action can be NOREWRITE, RESET or DROP.

        Default value: 0
        """
        return self.options['undefaction']

    def set_comment(self, comment):
        """
        Comments associated with this rewrite policy.

        Default value: 0
        """
        self.options['comment'] = comment

    def get_comment(self):
        """
        Comments associated with this rewrite policy.

        Default value: 0
        """
        return self.options['comment']

    def set_logaction(self, logaction):
        """
        The log action associated with the rewrite policy.

        Default value: 0
        """
        self.options['logaction'] = logaction

    def get_logaction(self):
        """
        The log action associated with the rewrite policy.

        Default value: 0
        """
        return self.options['logaction']

    def set_newname(self, newname):
        """
        The new name of the rewrite policy.

        Default value: 0
        Minimum length =  1.
        """
        self.options['newname'] = newname

    def get_newname(self):
        """
        The new name of the rewrite policy.

        Default value: 0
        Minimum length =  1.
        """
        return self.options['newname']

    def get_hits(self):
        """
        Number of hits.

        Default value: 0
        """
        return self.options['hits']

    def get_undefhits(self):
        """
        Number of undef hits.

        Default value: 0
        """
        return self.options['undefhits']

    def get_description(self):
        """
        Description of the policy.

        Default value: 0
        """
        return self.options['description']

    # Operations methods
    @staticmethod
    def get(nitro, rewritepolicy):
        """
        Use this API to fetch rewritepolicy resource of given name.
        """
        __rewritepolicy = NSRewritePolicy()
        __rewritepolicy.set_name(rewritepolicy.get_name())
        __rewritepolicy.get_resource(nitro)
        return __rewritepolicy

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured rewritepolicy resources.
        """
        __url = nitro.get_url() + NSRewritePolicy.get_resourcetype()
        __json_cspolicies = nitro.get(__url).get_response_field(NSRewritePolicy.get_resourcetype())
        __rewritepolicies = []
        for json_rewritepolicy in __json_cspolicies:
            __rewritepolicies.append(NSRewritePolicy(json_rewritepolicy))
        return __rewritepolicies

    @staticmethod
    def add(nitro, rewritepolicy):
        """
        Use this API to add rewritepolicy.
        """
        __rewritepolicy = NSRewritePolicy()
        __rewritepolicy.set_name(rewritepolicy.get_name())
        __rewritepolicy.set_rule(rewritepolicy.get_rule())
        __rewritepolicy.set_action(rewritepolicy.get_action())
        __rewritepolicy.set_undefaction(rewritepolicy.get_undefaction())
        __rewritepolicy.set_comment(rewritepolicy.get_comment())
        __rewritepolicy.set_logaction(rewritepolicy.get_logaction())
        return __rewritepolicy.add_resource(nitro)

    @staticmethod
    def delete(nitro, rewritepolicy):
        """
        Use this API to delete rewritepolicy of a given name.
        """
        __rewritepolicy = NSRewritePolicy()
        __rewritepolicy.set_name(rewritepolicy.get_name())
        nsresponse = __rewritepolicy.delete_resource(nitro)
        return nsresponse

    @staticmethod
    def update(nitro, rewritepolicy):
        """
        Use this API to update a rewritepolicy of a given name.
        """
        __rewritepolicy = NSRewritePolicy()
        __rewritepolicy.set_name(rewritepolicy.get_name())
        __rewritepolicy.set_rule(rewritepolicy.get_rule())
        __rewritepolicy.set_action(rewritepolicy.get_action())
        __rewritepolicy.set_undefaction(rewritepolicy.get_undefaction())
        __rewritepolicy.set_comment(rewritepolicy.get_comment())
        __rewritepolicy.set_logaction(rewritepolicy.get_logaction())
        return __rewritepolicy.update_resource(nitro)

    # No unset functionality for now.
    @staticmethod
    def rename(nitro, rewritepolicy):
        """
        Use this API to rename rewritepolicy.
        """
        __rewritepolicy = NSRewritePolicy()
        __rewritepolicy.set_name(rewritepolicy.get_name())
        __rewritepolicy.set_newname(rewritepolicy.get_newname())
        return __rewritepolicy.perform_operation(nitro, "rename")



from nsbaseresource import NSBaseResource

__author__ = 'ndenev@gmail.com'


class AuthTacacsAction(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(AuthTacacsAction, self).__init__()

        self.options = {'name': '',
                        'serverip': '',
                        'serverport': '',
                        'authtimeout': '',
                        'tacacssecret': '',
                        'authorization': '',
                        'accounting': '',
                        'auditfailedcmds': '',
                        'defaultauthenticationgroup': '',
                        # ReadOnly
                        'success': '',
                        'failure': ''}

        self.resourcetype = AuthTacacsAction.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "authenticationtacacsaction"

    def set_name(self, name):
        """
        Name for the TACACS+ profile (action). Must begin with a letter, number, or the underscore character (_), and must contain only letters,
        numbers, and the hyphen (-), period (.) pound (#), space ( ), at (@), equals (=), colon (:), and underscore characters.
        Cannot be changed after TACACS profile is created.
        Minimum length = 1
        """
        self.options['name'] = name

    def get_name(self):
        return self.options['name']

    def set_serverip(self, serverip):
        """
        IP address assigned to the TACACS+ server.
        """
        self.options['serverip'] = serverip

    def get_serverip(self):
        return self.options['serverip']

    def set_serverport(self, serverport):
        """
        Port number on which the TACACS+ server listens for connections.
        Default value: 49
        """
        self.options['serverport'] = serverport

    def get_serverport(self):
        return self.options['serverport']

    def set_authtimeout(self, authtimeout):
        """
        Number of seconds the NetScaler appliance waits for a response from the TACACS+ server.
        Default value: 3
        """
        self.options['authtimeout'] = authtimeout

    def get_authtimeout(self):
        return self.options['authtimeout']

    def set_tacacssecret(self, tacacssecret):
        """
        Key shared between the TACACS+ server and the NetScaler appliance. Required for allowing the NetScaler appliance to communicate with the TACACS+ server.
        Minimum length = 1
        """
        self.options['tacacssecret'] = tacacssecret

    def get_tacacssecret(self):
        return self.options['tacacssecret']

    def set_authorization(self, authorization):
        """
        Use streaming authorization on the TACACS+ server.
        """
        self.options['authorization'] = authorization

    def get_authorization(self):
        return self.options['authorization']

    def set_accounting(self, accounting):
        """
        Whether the TACACS+ server is currently accepting accounting messages.
        """
        self.options['accounting'] = accounting

    def get_accounting(self):
        return self.options['accounting']

    def set_auditfailedcmds(self, auditfailedcmds):
        """
        The state of the TACACS+ server that will receive accounting messages.
        """
        self.options['auditfailedcmds'] = auditfailedcmds

    def get_auditfailedcmds(self):
        return self.options['auditfailedcmds']

    def set_defaultauthenticationgroup(self, defaultauthenticationgroup):
        """
        This is the default group that is chosen when the authentication succeeds in addition to extracted groups.
        """
        self.options['defaultauthenticationgroup'] = defaultauthenticationgroup

    def get_defaultauthenticationgroup(self):
        return self.options['defaultauthenticationgroup']

    def get_success(self):
        return self.options['success']

    def get_failure(self):
        return self.options['failure']

    # Operations methods
    @staticmethod
    def get(nitro, tacacsaction):
        """
        Use this API to fetch a configured authenticationtacacsaction resource by name.
        """
        __tacacsaction = AuthTacacsAction()
        __tacacsaction.set_name(tacacsaction.get_name())
        __tacacsaction.get_resource(nitro)
        return __tacacsaction

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured authenticationtacacsaction resources.
        """
        __url = nitro.get_url() + AuthTacacsAction.get_resourcetype()
        __json_tacacsactions = nitro.get(__url).get_response_field(AuthTacacsAction.get_resourcetype())
        __tacacsactions = []
        for json_tacacsaction in __json_tacacsactions:
            __tacacsactions.append(AuthTacacsAction(json_tacacsaction))
        return __tacacsactions

    @staticmethod
    def add(nitro, tacacsaction):
        """
        Use this API to add authenticationtacacsaction.
        """
        __tacacsaction = AuthTacacsAction()
        __tacacsaction.set_name(tacacsaction.get_name())
        __tacacsaction.set_serverip(tacacsaction.get_serverip())
        __tacacsaction.set_serverport(tacacsaction.get_serverport())
        __tacacsaction.set_authtimeout(tacacsaction.get_authtimeout())
        __tacacsaction.set_tacacssecret(tacacsaction.get_tacacssecret())
        __tacacsaction.set_authorization(tacacsaction.get_authorization())
        __tacacsaction.set_accounting(tacacsaction.get_accounting())
        __tacacsaction.set_auditfailedcmds(tacacsaction.get_auditfailedcmds())
        __tacacsaction.set_defaultauthenticationgroup(tacacsaction.get_defaultauthenticationgroup())
        return __tacacsaction.add_resource(nitro)

    @staticmethod
    def delete(nitro, tacacsaction):
        """
        Use this API to delete authenticationtacacsaction of a given name.
        """
        __tacacsaction = AuthTacacsAction()
        __tacacsaction.set_name(tacacsaction.get_name())
        nsresponse = __tacacsaction.delete_resource(nitro)
        return nsresponse

    @staticmethod
    def update(nitro, tacacsaction):
        """
        Use this API to update a rewriteaction of a given name.
        """
        __tacacsaction = AuthTacacsAction()
        __tacacsaction.set_name(tacacsaction.get_name())
        __tacacsaction.set_serverip(tacacsaction.get_serverip())
        __tacacsaction.set_serverport(tacacsaction.get_serverport())
        __tacacsaction.set_authtimeout(tacacsaction.get_authtimeout())
        __tacacsaction.set_tacacssecret(tacacsaction.get_tacacssecret())
        __tacacsaction.set_authorization(tacacsaction.get_authorization())
        __tacacsaction.set_accounting(tacacsaction.get_accounting())
        __tacacsaction.set_auditfailedcmds(tacacsaction.get_auditfailedcmds())
        __tacacsaction.set_defaultauthenticationgroup(tacacsaction.get_defaultauthenticationgroup())
        return __tacacsaction.update_resource(nitro)

    # No unset functionality for now.



from nsbaseresource import NSBaseResource
__author__ = 'vlazarenko'


class NSLBVServerServiceBinding(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSLBVServerServiceBinding, self).__init__()
        self.options = {
            'servicename': '',
            'ipv46': '',
            'port': '',
            'servicetype': '',
            'curstate': '',
            'weight': '',
            'dynamicweight': '',
            'cookieipport': '',
            'vserverid': '',
            'name': '',
            'servicegroupname': ''
        }

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

        self.resourcetype = NSLBVServerServiceBinding.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "lbvserver_service_binding"

    # Getters and setters for configurable options
    def set_servicename(self, servicename):
        """
        The service name bound to the selected load balancing virtual server.
        Default value: 0
        Minimum length =  1.
        """
        self.options['servicename'] = servicename

    def get_servicename(self):
        """
        The service name bound to the selected load balancing virtual server.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['servicename']

    def get_ipv46(self):
        """
        The IP address of the virtual server.
        Default value: 0
        """
        return self.options['ipv46']

    def get_port(self):
        """
        The IP address of the virtual server.
        Default value: 0
        """
        return self.options['port']

    def get_servicetype(self):
        """
        The service type.
        Default value: 0
        """
        return self.options['servicetype']

    def get_curstate(self):
        """
        Current LB vserver state.
        Default value: 0
        """
        return self.options['curstate']

    def set_weight(self, weight):
        """
        The weight for the specified service.
        Default value: 0
        Minimum value =  1
        Maximum value =  100
        """
        self.options['weight'] = weight

    def get_weight(self):
        """
        The weight for the specified service.
        Default value: 0
        Minimum value =  1
        Maximum value =  100
        """
        return self.options['weight']

    def get_dynamicweight(self):
        """
        Dynamic weight.
        Default value: 0
        """
        return self.options['dynamicweight']

    def get_cookieipport(self):
        """
        Encryped Ip address and port of the service that is inserted into the set-cookie http header.
        Default value: 0
        """
        return self.options['cookieipport']

    def get_vserverid(self):
        """
        Vserver Id.
        Default value: 0
        """
        return self.options['vserverid']

    def set_name(self, name):
        """
        The virtual server name to which the service is bound.
        Default value: 0
        Minimum length =  1.
        """
        self.options['name'] = name

    def get_name(self):
        """
        The virtual server name to which the service is bound.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['name']

    def set_servicegroupname(self, servicegroupname):
        """
        The name of the service group that is bound.
        Default value: 0
        Minimum length =  1.
        """
        self.options['servicegroupname'] = servicegroupname

    def get_servicegroupname(self):
        """
        The name of the service group that is bound.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['servicegroupname']

    # Operations methods
    @staticmethod
    def get(nitro, vserver_service_binding):
        """
        Use this API to fetch configured vserver_service_binding resources of a given name.
        """
        __url = nitro.get_url() + NSLBVServerServiceBinding.get_resourcetype() + \
            "/" + vserver_service_binding.get_name()
        __json_services = nitro.get(__url).get_response_field(
            NSLBVServerServiceBinding.get_resourcetype())
        __services = []
        for json_service in __json_services:
            __services.append(NSLBVServerServiceBinding(json_service))
        return __services


    @staticmethod
    def add(nitro, vserver_service_binding):
        """
        Use this API to add vserver_service_binding.
        """
        __vserver_service_binding = NSLBVServerServiceBinding()
        __vserver_service_binding.set_name(vserver_service_binding.get_name())
        __vserver_service_binding.set_servicename(
            vserver_service_binding.get_servicename())
        __vserver_service_binding.set_weight(
            vserver_service_binding.get_weight())
        __vserver_service_binding.set_servicegroupname(
            vserver_service_binding.get_servicegroupname())
        return __vserver_service_binding.update_resource(nitro)

    @staticmethod
    def delete(nitro, vserver_service_binding):
        """
        Use this API to delete vserver_service_binding of a given name.
        """
        __vserver_service_binding = NSLBVServerServiceBinding()
        __vserver_service_binding.set_name(vserver_service_binding.get_name())
        __vserver_service_binding.set_servicename(
            vserver_service_binding.get_servicename())
        __vserver_service_binding.set_servicegroupname(
            vserver_service_binding.get_servicegroupname())
        nsresponse = __vserver_service_binding.delete_resource(nitro)
        return nsresponse



from nsbaseresource import NSBaseResource

__author__ = 'Aleksandar Topuzovic'

class NSHardware(NSBaseResource):

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """

                super(NSHardware, self).__init__()

                self.options = {
                        'hwdescription': '',
                        'sysid': '',
                        'manufactureday': '',
                        'manufacturemonth': '',
                        'manufactureyear': '',
                        'cpufrequncy': '',
                        'hostid': '',
                        'host': '',
                        'serialno': '',
                        'encodedserialno': ''
                }

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options.keys():
                                        self.options[key] = json_data[key]

                self.resourcetype = NSHardware.get_resourcetype()

        def get_hwdescription(self):
            return self.options['hwdescription']

        def get_sysid(self):
            return self.options['sysid']

        def get_manufactureday(self):
            return self.options['manufactureday']

        def get_manufacturemonth(self):
            return self.options['manufacturemonth']

        def get_manufactureyear(self):
            return self.options['manufactureyear']

        def get_cpufrequncy(self):
            return self.options['cpufrequncy']

        def get_hostid(self):
            return self.options['hostid']

        def get_host(self):
            return self.options['host']

        def get_serialno(self):
            return self.options['serialno']

        def get_encodedserialno(self):
            return self.options['encodedserialno']

        @staticmethod
        def get_resourcetype():
                return "nshardware"

        @staticmethod
        def get(nitro):
            __url = nitro.get_url() + NSHardware.get_resourcetype()
            __json_nshardware = nitro.get(__url).get_response_field(NSHardware.get_resourcetype())
            return NSHardware(__json_nshardware)



from nsbaseresource import NSBaseResource
__author__ = 'Aleksandar Topuzovic'


class NSServiceLBMonitorBinding(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSServiceLBMonitorBinding, self).__init__()
        self.options = {'policyname': '',
                        'name': '',
                        'monitor_name': '',
                        'monstatparam2': '',
                        'monstatcode': '',
                        'failedprobes': '',
                        'monstatparam3': '',
                        'totalprobes': '',
                        'responsetime': '',
                        'monstatparam1': '',
                        'monitor_state': '',
                        'monstate': '',
                        'totalfailedprobes': '',
                        'dup_weight': ''}

        self.resourcetype = NSServiceLBMonitorBinding.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        """
        Binding object showing the lbmonitor that can be bound to service.
        """
        return "service_lbmonitor_binding"

    # Read/write properties
    def set_policyname(self, policyname):
        """
        The DoS protection policy name must be bound to the service. Also, for DoS protection to work on a service, an appropriate policy must be bound to the service.
        Minimum length = 1
        """
        self.options['policyname'] = policyname

    def get_policyname(self):
        """
        The DoS protection policy name must be bound to the service. Also, for DoS protection to work on a service, an appropriate policy must be bound to the service.
        Minimum length = 1
        """
        return self.options['policyname']

    def set_name(self, name):
        """
        The name of the service to which the policy will be bound.
        Minimum length = 1
        """
        self.options['name'] = name

    def get_name(self):
        """
        The name of the service to which the policy will be bound.
        Minimum length = 1
        """
        return self.options['name']

    def set_monitor_name(self, monitorname):
        """
        The monitor Names.
        """
        self.options['monitor_name'] = monitorname

    def get_monitor_name(self):
        """
        The monitor Names.
        """
        return self.options['monitor_name']

    # Read only properties
    def get_monstatparam2(self):
        """
        Second parameter for use with message code.
        """
        return self.options['monstatparam2']

    def get_monstatcode(self):
        """
        The code indicating the monitor response.
        """
        return self.options['monstatcode']

    def get_failedprobes(self):
        """
        Number of the current failed monitoring probes.
        """
        return self.options['failedprobes']

    def get_monitorname(self):
        """
        Third parameter for use with message code.
        """
        return self.options['monstatparam3']

    def get_totalprobes(self):
        """
        The total number of probs sent.
        """
        return self.options['totalprobes']

    def get_responsetime(self):
        """
        Response time of this monitor.
        """
        return self.options['responsetime']

    def get_monstatparam1(self):
        """
        First parameter for use with message code.
        """
        return self.options['monstatparam1']

    def get_monitor_state(self):
        """
        The running state of the monitor on this service.
        """
        return self.options['monitor_state']

    def get_monstate(self):
        """
        The configured state (enable/disable) of the monitor on this server.
        """
        return self.options['monstate']

    def get_totalfailedprobes(self):
        """
        The total number of failed probs.
        """
        return self.options['totalfailedprobes']

    def get_dup_weight(self):
        """
        The weight of the monitor.
        """
        return self.options['dup_weight']

    @staticmethod
    def get(nitro, servicelbmonitorbinding):
        """
        Use this API to fetch service monitor binding resource of given name.
        """
        __url = nitro.get_url() + NSServiceLBMonitorBinding.get_resourcetype() + "/" + servicelbmonitorbinding.get_name()
        __json_servicelbmonitorbindings = nitro.get(__url).get_response_field(NSServiceLBMonitorBinding.get_resourcetype())
        __servicelbmonitorbindings = []
        for json_servicelbmonitorbinding in __json_servicelbmonitorbindings:
            __servicelbmonitorbindings.append(NSServiceLBMonitorBinding(json_servicelbmonitorbinding))
        return __servicelbmonitorbindings



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSVLAN(NSBaseResource):

    # General Netscaler configuration object

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """

        super(NSVLAN, self).__init__()

        self.options = {
            'id': '',
            'ipv6dynamicrouting': '',
            'rnat': '',
            'portbitmap': '',
            'tagbitmap': '',
            'ifaces': '',
            'tagifaces': '',
        }

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

        self.resourcetype = NSVLAN.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "vlan"

    def set_id(self, id):
        self.options['id'] = id

    def get_id(self):
        return self.options['id']

    def set_ipv6dynamicrouting(self, ipv6dynamicrouting):
        self.options['ipv6dynamicrouting'] = ipv6dynamicrouting

    def get_ipv6dynamicrouting(self):
        return self.options['ipv6dynamicrouting']

    def get_rnat(self):
        return self.options['rnat']

    def get_portbitmap(self):
        return self.options['portbitmap']

    def get_tagbitmap(self):
        return self.options['tagbitmap']

    def get_ifaces(self):
        return self.options['ifaces']

    def get_tagifaces(self):
        return self.options['tagifaces']

    @staticmethod
    def add(nitro, vlan):
        __vlan = NSVLAN()
        __vlan.set_id(vlan.get_id())
        __vlan.set_ipv6dynamicrouting(vlan.get_ipv6dynamicrouting())
        return __vlan.add_resource(nitro)

    @staticmethod
    def update(nitro, vlan):
        __vlan = NSVLAN()
        __vlan.set_id(vlan.get_id())
        __vlan.set_ipv6dynamicrouting(vlan.get_ipv6dynamicrouting())
        return __vlan.update_resource(nitro)

    @staticmethod
    def delete(nitro, vlan):
        __vlan = NSVLAN()
        __vlan.set_id(vlan.get_id())
        nsresponse = __vlan.delete_resource(nitro, object_name=__vlan.get_id())
        return nsresponse

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all vlan resources that are configured on netscaler.
        """
        __url = nitro.get_url() + NSVLAN.get_resourcetype()
        __json_vlans = nitro.get(__url).get_response_field(NSVLAN.get_resourcetype())
        __vlans = []
        for json_vlan in __json_vlans:
            __vlans.append(NSVLAN(json_vlan))
        return __vlans



import os
from setuptools import setup,find_packages

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name = "nsnitro",
    version = "1.0.33",
    author = "Vladimir Lazarenko",
    author_email = "vllazarenko@ebay.com",
    description = ("A simple library to control Citrix Netscaler 9.2+ with NITRO API."),
    license = "GPL",
    keywords = "citrix netscaler nitro api nsnitro",
    url = "http://pypi.python.org/pypi/nsnitro",
    packages=["nsnitro"] + [os.path.join("nsnitro",a) for a in find_packages("nsnitro")],
    long_description=read('README'),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Topic :: Utilities",
        "License :: OSI Approved :: Python Software Foundation License",
    ],
    install_requires=[
        "httplib2",
        "argparse",
    ],
    scripts=[
        'bin/nsnitrocmd.py'
    ],
)



__author__ = 'vllazarenko'

from nsbaseresource import NSBaseResource


class NSFeature(NSBaseResource):

    # Configuration for NS Features.

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSFeature, self).__init__()
        self.options = {'feature': '',
                        'reqfeature': '',
                        'wl': '',
                        'sp': '',
                        'lb': '',
                        'cs': '',
                        'cr': '',
                        'sc': '',
                        'cmp': '',
                        'pq': '',
                        'ssl': '',
                        'gslb': '',
                        'hdosp': '',
                        'cf': '',
                        'ic': '',
                        'sslvpn': '',
                        'aaa': '',
                        'ospf': '',
                        'rip': '',
                        'bgp': '',
                        'rewrite': '',
                        'ipv6pt': '',
                        'appfw': '',
                        'responder': '',
                        'htmlinjection': '',
                        'push': ''}

        self.resourcetype = NSFeature.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "nsfeature"

    def set_feature(self, feature):
        self.options['feature'] = feature

    def get_feature(self):
        return self.options['feature']

    def get_reqfeature(self):
        return self.options['reqfeature']

    def get_wl(self):
        return self.options['wl']

    def get_sp(self):
        return self.options['sp']

    def get_lb(self):
        return self.options['lb']

    def get_cs(self):
        return self.options['cs']

    def get_cr(self):
        return self.options['cr']

    def get_sc(self):
        return self.options['sc']

    def get_cmp(self):
        return self.options['cmp']

    def get_pq(self):
        return self.options['pq']

    def get_ssl(self):
        return self.options['ssl']

    def get_gslb(self):
        return self.options['gslb']

    def get_hdosp(self):
        return self.options['hdosp']

    def get_cf(self):
        return self.options['cf']

    def get_ic(self):
        return self.options['ic']

    def get_sslvpn(self):
        return self.options['sslvpn']

    def get_aaa(self):
        return self.options['aaa']

    def get_ospf(self):
        return self.options['ospf']

    def get_rip(self):
        return self.options['rip']

    def get_bgp(self):
        return self.options['bgp']

    def get_rewrite(self):
        return self.options['rewrite']

    def get_ipv6pt(self):
        return self.options['ipv6pt']

    def get_appfw(self):
        return self.options['appfw']

    def get_responder(self):
        return self.options['responder']

    def get_htmlinjection(self):
        return self.options['htmlinjection']

    def get_push(self):
        return self.options['push']

    # Operations methods
    @staticmethod
    def disable(nitro, resource):
        """
        Use this API to disable server.
        """
        __resource = NSFeature()
        __resource.set_feature(resource.get_feature())
        return __resource.perform_operation(nitro, "disable")

    @staticmethod
    def enable(nitro, resource):
        """
        Use this API to enable server.
        """
        __resource = NSFeature()
        __resource.set_feature(resource.get_feature())
        return __resource.perform_operation(nitro, "enable")



from nsnitro import NSNitro
from nsutil import NSNitroResponse
from nsresources import *
from nsexceptions import *
import nsresources
import nsexceptions

__all__ = ['NSNitro', 'NSNitroResponse'] + nsresources.__all__ + nsexceptions.__all__



from nsbaseresource import NSBaseResource

__author__ = 'ndenev'


class NSRoute(NSBaseResource):

    # General Netscaler configuration object

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """

        super(NSRoute, self).__init__()

        self.options = {'network': '',
                        'netmask': '',
                        'gateway': '',
                        'cost': '',
                        'td': '',
                        'distance': '',
                        'cost1': '',
                        'weight': '',
                        'advertise': '',
                        'protocol': '',
                        'msr': '',
                        'monitor': '',
                        'routetype': '',
                        'detail': '',
                        'gatewayname': '',
                        'type': '',
                        'dynamic': '',
                        'Static': '',
                        'permanent': '',
                        'direct': '',
                        'nat': '',
                        'lbroute': '',
                        'adv': '',
                        'tunnel': '',
                        'data': '',
                        'data0': '',
                        'flags': '',
                        'routeowners': '',
                        'retain': '',
                        'ospf': '',
                        'isis': '',
                        'rip': '',
                        'bgp': '',
                        'dhcp': '',
                        'advospf': '',
                        'advisis': '',
                        'advrip': '',
                        'advbgp': '',
                        'state': '',
                        'totalprobes': '',
                        'totalfailedprobes': '',
                        'failedprobes': '',
                        'monstatcode': '',
                        'monstatparam1': '',
                        'monstatparam2': '',
                        'monstatparam3': '',
                        '__count': ''}

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

        self.resourcetype = NSRoute.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "route"

    def set_network(self, network):
        self.options['network'] = network

    def get_network(self):
        return self.options['network']

    def set_netmask(self, netmask):
        self.options['netmask'] = netmask

    def get_netmask(self):
        return self.options['netmask']

    def set_gateway(self, gateway):
        self.options['gateway'] = gateway

    def get_gateway(self):
        return self.options['gateway']

    def set_cost(self, cost):
        self.options['cost'] = cost

    def get_cost(self):
        return self.options['cost']

    def set_td(self, td):
        self.options['td'] = td

    def get_td(self):
        return self.options['td']

    def set_distance(self, distance):
        self.options['distance'] = distance

    def get_distance(self):
        return self.options['distance']

    def set_cost1(self, cost1):
        self.options['cost1'] = cost1

    def get_cost1(self):
        return self.options['cost1']

    def set_weight(self, weight):
        self.options['weight'] = weight

    def get_weight(self):
        return self.options['weight']

    def set_advertise(self, advertise):
        self.options['advertise'] = advertise

    def get_advertise(self):
        return self.options['advertise']

    def set_protocol(self, protocol):
        self.options['protocol'] = protocol

    def get_protocol(self):
        return self.options['protocol']

    def set_msr(self, msr):
        self.options['msr'] = msr

    def get_msr(self):
        return self.options['msr']

    def set_monitor(self, monitor):
        self.options['monitor'] = monitor

    def get_monitor(self):
        return self.options['monitor']

    def set_routetype(self, routetype):
        self.options['routetype'] = routetype

    def get_routetype(self):
        return self.options['routetype']

    def set_detail(self, detail):
        self.options['detail'] = detail

    def get_detail(self):
        return self.options['detail']

    def get_gatewayname(self):
        return self.options['gatewayname']

    def get_type(self):
        return self.options['type']

    def get_dynamic(self):
        return self.options['dynamic']

    def get_static(self):
        return self.options['Static']

    def get_permanent(self):
        return self.options['permanent']

    def get_direct(self):
        return self.options['direct']

    def get_nat(self):
        return self.options['nat']

    def get_lbroute(self):
        return self.options['lbroute']

    def get_adv(self):
        return self.options['adv']

    def get_tunnel(self):
        return self.options['tunnel']

    def get_data(self):
        return self.options['data']

    def get_data0(self):
        return self.options['data0']

    def get_flags(self):
        return self.options['flags']

    def get_routeowners(self):
        return self.options['routeowners']

    def get_retain(self):
        return self.options['retain']

    def get_ospf(self):
        return self.options['ospf']

    def get_isis(self):
        return self.options['isis']

    def get_rip(self):
        return self.options['rip']

    def get_bgp(self):
        return self.options['bgp']

    def get_dhcp(self):
        return self.options['dhcp']

    def get_advospf(self):
        return self.options['advospf']

    def get_advisis(self):
        return self.options['advisis']

    def get_advrip(self):
        return self.options['advrip']

    def get_advbgp(self):
        return self.options['advbgp']

    def get_state(self):
        return self.options['state']

    def get_totalprobes(self):
        return self.options['totalprobes']

    def get_totalfailedprobes(self):
        return self.options['totalfailedprobes']

    def get_failedprobes(self):
        return self.options['failedprobes']

    def get_monstatcode(self):
        return self.options['monstatcode']

    def get_monstatparam1(self):
        return self.options['monstatparam1']

    def get_monstatparam2(self):
        return self.options['monstatparam2']

    def get_monstatparam3(self):
        return self.options['monstatparam3']

    @staticmethod
    def add(nitro, route):
        __route = NSRoute()
        __route.set_network(route.get_network())
        __route.set_netmask(route.get_netmask())
        __route.set_gateway(route.get_gateway())
        __route.set_cost(route.get_cost())
        __route.set_td(route.get_td())
        __route.set_distance(route.get_distance())
        __route.set_cost1(route.get_cost1())
        __route.set_weight(route.get_weight())
        __route.set_advertise(route.get_advertise())
        __route.set_protocol(route.get_protocol())
        __route.set_msr(route.get_msr())
        __route.set_monitor(route.get_monitor())
        return __route.add_resource(nitro)

    @staticmethod
    def clear(nitro, route):
        __route = NSRoute()
        __route.set_routetype(route.get_routetype())
        return __route.perform_operation(nitro, "clear")

    @staticmethod
    def delete(nitro, route):
        __route = NSRoute()
        __route.set_netmask(route.get_netmask())
        __route.set_gateway(route.get_gateway())
        __route.set_td(route.get_td())
        nsresponse = __route.delete_resource(nitro, object_name=route.get_network())
        return nsresponse

    @staticmethod
    def update(nitro, route):
        __route = NSRoute()
        __route.set_network(route.get_network())
        __route.set_netmask(route.get_netmask())
        __route.set_gateway(route.get_gateway())
        __route.set_td(route.get_td())
        __route.set_distance(route.get_distance())
        __route.set_cost1(route.get_cost1())
        __route.set_weight(route.get_weight())
        __route.set_advertise(route.get_advertise())
        __route.set_protocol(route.get_protocol())
        __route.set_msr(route.get_msr())
        __route.set_monitor(route.get_monitor())
        return __route.update_resource(nitro)

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all nsroute resources that are configured on netscaler.
        """
        __url = nitro.get_url() + NSRoute.get_resourcetype()
        __json_routes = nitro.get(__url).get_response_field(NSRoute.get_resourcetype())
        __routes = []
        for json_route in __json_routes:
            __routes.append(NSRoute(json_route))
        return __routes

    # No route count functionality for now.
    # No unset functionality for now.



from nsbaseresource import NSBaseResource

__author__ = 'Aleksandar Topuzovic'


class NSRunningConfig(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """

        super(NSRunningConfig, self).__init__()

        self.options = {'withdefaults': '',
                        'response': ''}

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

        self.resourcetype = NSRunningConfig.get_resourcetype()

    def get_withdefaults(self):
        return self.options['withdefaults']

    def set_withdefaults(self, withdefaults):
        self.options['withdefaults'] = withdefaults

    def get_response(self):
        return self.options['response']

    @staticmethod
    def get_resourcetype():
        return "nsrunningconfig"

    @staticmethod
    def get(nitro):
        __url = nitro.get_url() + NSRunningConfig.get_resourcetype()
        __json_nsversion = nitro.get(__url).get_response_field(NSRunningConfig.get_resourcetype())
        return NSRunningConfig(__json_nsversion)



from nsbaseresource import NSBaseResource

__author__ = 'Aleksandar Topuzovic'

class NSHostname(NSBaseResource):

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """

                super(NSHostname, self).__init__()

                self.options = {
                        'hostname': '',
                }

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options.keys():
                                        self.options[key] = json_data[key]

                self.resourcetype = NSHostname.get_resourcetype()

        def get_hostname(self):
            return self.options['hostname']

        @staticmethod
        def get_resourcetype():
                return "nshostname"

        @staticmethod
        def get(nitro):
            __url = nitro.get_url() + NSHostname.get_resourcetype()
            __json_nshostname = nitro.get(__url).get_response_field(NSHostname.get_resourcetype())
            if isinstance(__json_nshostname, list):
                    return NSHostname(__json_nshostname[0])
            return NSHostname(__json_nshostname)
